@@ -142,60 +142,60 @@ static const struct tegra_emc_table paz00_emc_tables_Hynix_333Mhz[] =
 	}
 };
 
+/* MT47H128M8CF-25:H */
 static const struct tegra_emc_table paz00_emc_tables_Micron_333Mhz[] = {
 	{
 
-/*** FIXME ! should be for 166 MHz ****/
-		.rate = 190000,   /* SDRAM frequency */
-		.regs = {
-			0x0000000b,   /* RC */
-			0x00000026,   /* RFC */
-			0x00000008,   /* RAS */
-			0x00000003,   /* RP */
-			0x00000004,   /* R2W */
-			0x00000004,   /* W2R */
-			0x00000002,   /* R2P */
-			0x0000000b,   /* W2P */
-			0x00000003,   /* RD_RCD */
-			0x00000003,   /* WR_RCD */
-			0x00000002,   /* RRD */
-			0x00000001,   /* REXT */
-			0x00000003,   /* WDV */
-			0x00000004,   /* QUSE */
-			0x00000003,   /* QRST */
-			0x00000009,   /* QSAFE */
-			0x0000000c,   /* RDV */
-			0x0000059f,   /* REFRESH */
-			0x00000000,   /* BURST_REFRESH_NUM */
-			0x00000003,   /* PDEX2WR */
-			0x00000003,   /* PDEX2RD */
-			0x00000003,   /* PCHG2PDEN */
-			0x00000003,   /* ACT2PDEN */
-			0x00000001,   /* AR2PDEN */
-			0x0000000a,   /* RW2PDEN */
-			0x000000c8,   /* TXSR */
-			0x00000003,   /* TCKE */
-			0x00000007,   /* TFAW */
-			0x00000004,   /* TRPAB */
-			0x0000000f,   /* TCLKSTABLE */
-			0x00000002,   /* TCLKSTOP */
-			0x00000000,   /* TREFBW */
-			0x00000000,   /* QUSE_EXTRA */
-			0x00000003,   /* FBIO_CFG6 */
-			0x00000000,   /* ODT_WRITE */
-			0x00000000,   /* ODT_READ */
-			0x00000083,   /* FBIO_CFG5 */
-			0xa06204ae,   /* CFG_DIG_DLL */
-			0x007e4010,   /* DLL_XFORM_DQS */
-			0x00000000,   /* DLL_XFORM_QUSE */
-			0x00000000,   /* ZCAL_REF_CNT */
-			0x00000000,   /* ZCAL_WAIT_CNT */
-			0x00000000,   /* AUTO_CAL_INTERVAL */
-			0x00000000,   /* CFG_CLKTRIM_0 */
-			0x00000000,   /* CFG_CLKTRIM_1 */
-			0x00000000,   /* CFG_CLKTRIM_2 */
-		}
-	}, {
+//		.rate = 166500,   /* SDRAM frequency */
+//		.regs = {
+//			0x0000000b,   /* RC */
+//			0x00000026,   /* RFC */
+//			0x00000008,   /* RAS */
+//			0x00000003,   /* RP */
+//			0x00000004,   /* R2W */
+//			0x00000004,   /* W2R */
+//			0x00000002,   /* R2P */
+//			0x0000000b,   /* W2P */
+//			0x00000003,   /* RD_RCD */
+//			0x00000003,   /* WR_RCD */
+//			0x00000002,   /* RRD */
+//			0x00000001,   /* REXT */
+//			0x00000003,   /* WDV */
+//			0x00000004,   /* QUSE */
+//			0x00000003,   /* QRST */
+//			0x00000009,   /* QSAFE */
+//			0x0000000c,   /* RDV */
+//			0x0000059f,   /* REFRESH */
+//			0x00000000,   /* BURST_REFRESH_NUM */
+//			0x00000003,   /* PDEX2WR */
+//			0x00000003,   /* PDEX2RD */
+//			0x00000003,   /* PCHG2PDEN */
+//			0x00000003,   /* ACT2PDEN */
+//			0x00000001,   /* AR2PDEN */
+//			0x0000000a,   /* RW2PDEN */
+//			0x000000c8,   /* TXSR */
+//			0x00000003,   /* TCKE */
+//			0x00000007,   /* TFAW */
+//			0x00000004,   /* TRPAB */
+//			0x0000000f,   /* TCLKSTABLE */
+//			0x00000002,   /* TCLKSTOP */
+//			0x00000000,   /* TREFBW */
+//			0x00000000,   /* QUSE_EXTRA */
+//			0x00000003,   /* FBIO_CFG6 */
+//			0x00000000,   /* ODT_WRITE */
+//			0x00000000,   /* ODT_READ */
+//			0x00000083,   /* FBIO_CFG5 */
+//			0xa06204ae,   /* CFG_DIG_DLL */
+//			0x007e4010,   /* DLL_XFORM_DQS */
+//			0x00000000,   /* DLL_XFORM_QUSE */
+//			0x00000000,   /* ZCAL_REF_CNT */
+//			0x00000000,   /* ZCAL_WAIT_CNT */
+//			0x00000000,   /* AUTO_CAL_INTERVAL */
+//			0x00000000,   /* CFG_CLKTRIM_0 */
+//			0x00000000,   /* CFG_CLKTRIM_1 */
+//			0x00000000,   /* CFG_CLKTRIM_2 */
+//		}
+//	}, // {
 		.rate = 333000,   /* SDRAM frequency */
 		.regs = {
 			0x00000014,   /* RC */
@@ -244,8 +244,8 @@ static const struct tegra_emc_table paz00_emc_tables_Micron_333Mhz[] = {
 			0x00000000,   /* CFG_CLKTRIM_0 */
 			0x00000000,   /* CFG_CLKTRIM_1 */
 			0x00000000,   /* CFG_CLKTRIM_2 */
-		}
-	}
+		},
+	},
 };
 
 struct tegra_board_emc_table paz00_emc[] = {
@@ -279,16 +279,11 @@ void __init paz00_emc_init(void)
 	ram_id = (reg & RAM_ID_MASK) >> RAM_CODE_SHIFT;
 
 	pr_warning("EMC table: ramd_id: %d, tegra_sku_id %d\n", ram_id, tegra_sku_id);
-/* FIXME:
-   Add timings for Micron chips @ 166 MHz (no switching is done for Micron so far)
-   Hynix may work, but I can test it because of missing hardware, so disable it
-   all together */
+
 	if(ram_id <= ARRAY_SIZE(paz00_emc))
 	{
 		pr_warning("EMC table: using %s\n", paz00_emc[ram_id].name);
-#if 0
 		tegra_init_emc(paz00_emc[ram_id].table, paz00_emc[ram_id].table_size);
-#endif
 	} else {
 		pr_warning("EMC table: unknown RAM ID - Please report !!!\n");
 		tegra_init_emc(NULL, 0);
