@@ -37,6 +37,7 @@
 #include <mach/fb.h>
 
 #include "devices.h"
+#include "gpio-names.h"
 #include "board-paz00.h"
 
 static int paz00_backlight_init(struct device *dev)
@@ -121,84 +122,7 @@ static struct platform_device paz00_backlight_device = {
 
 static int paz00_panel_enable(void)
 {
-/*	static struct regulator *reg = NULL;
-	int ret = 0;
-
-	pr_warning(">>> panel_enable\n");
-
-	reg = regulator_get(NULL, "avdd_lvds");
-	if (IS_ERR(reg)) {
-		pr_warning("Couldn't get regulator avdd_lvds\n");
-		return -1;
-	}
-
-	if (regulator_is_enabled(reg)) {
-		pr_warning("regulator avdd_lvds already enabled\n");
-	} else {
-
-		ret = regulator_set_voltage(reg, 3300000, 3300000);
-		if (ret) {
-			pr_warning("Couldn't set regulator voltage add_lvds\n");
-			return -1;
-		}
-
-		ret = regulator_enable(reg);
-		if (ret)
-			pr_warning("Couldn't enable regualtor avdd_lvds\n");
-	}
-
-	ret = gpio_request(PAZ00_LVDS_SHUTDOWN, "lvds_shdn");
-	if (ret) {
-		pr_warning("could not request PAZ00_LVDS_SHUTDOWN gpio\n");
-		return ret;
-	}
-
-	ret = gpio_direction_output(PAZ00_LVDS_SHUTDOWN, 1);
-	if (ret) {
-		pr_warning("could not set output direction of lvds_shutdown\n");
-		return ret;
-	}
-*/
 	gpio_set_value(TEGRA_LVDS_SHUTDOWN, 1);
-
-/* Backlight */
-/*
-	pr_warning(">>> backlight_enable\n");
-
-	reg = regulator_get(NULL, "vddio_lcd");
-	if (IS_ERR(reg)) {
-		pr_warning("Couldn't get regulator vddio_lcd\n");
-		return -1;
-	}
-
-	if (regulator_is_enabled(reg)) {
-		pr_warning("regulator vddio_lcd already enabled\n");
-	} else {
-		ret = regulator_set_voltage(reg, 1800000, 1800000);
-		if (ret) {
-			pr_warning("Couldn't set regulator voltage vddio_lcd\n");
-			return -1;
-		}
-
-		ret = regulator_enable(reg);
-		if (ret)
-			pr_warning("Couldn't enable regualtor vddio_lcd\n");
-	}
-
-	ret = gpio_request(PAZ00_BACKLIGHT, "blacklight_enable");
-	if (ret) {
-		pr_warning("could not request paz00_bl_enb gpio\n");
-		return ret;
-	}
-
-	ret = gpio_direction_output(PAZ00_BACKLIGHT, 1);
-	if (ret) {
-		pr_warning("could not set output direction of lvds_shutdown\n");
-		return ret;
-	}
-
-	gpio_set_value(PAZ00_BACKLIGHT, 1);
-*/
 	return 0;
 }
 
@@ -217,7 +141,6 @@ static int paz00_hdmi_enable(void)
 	if (WARN_ON(!paz00_hdmi_reg || !paz00_hdmi_pll))
 		return -ENODEV;
 
-//	gpio_set_value(paz00_hdmi_enb, 1);
 	regulator_enable(paz00_hdmi_reg);
 	regulator_enable(paz00_hdmi_pll);
 	return 0;
@@ -229,7 +152,6 @@ static int paz00_hdmi_disable(void)
 	if (WARN_ON(!paz00_hdmi_reg || !paz00_hdmi_pll))
 		return -ENODEV;
 
-//	gpio_set_value(paz00_hdmi_enb, 0);
 	regulator_disable(paz00_hdmi_reg);
 	regulator_disable(paz00_hdmi_pll);
 	return 0;
@@ -251,7 +173,7 @@ static struct resource paz00_disp1_resources[] = {
 	{
 		.name	= "fbmem",
 		.start	= 0x1fd95000,
-		.end	= 0x1fd95000 + 0x26B000 - 1,   // 2.4 MB @ 509 MB
+		.end	= 0x1fd95000 + 0x26B000 - 1,	/* 2.4 MB @ 509 MB */
 		.flags	= IORESOURCE_MEM,
 	},
 };
@@ -388,7 +310,7 @@ static struct nvmap_platform_carveout paz00_carveouts[] = {
 	[1] = {
 		.name		= "generic-0",
 		.usage_mask	= NVMAP_HEAP_CARVEOUT_GENERIC,
-		.base		= 0x1C000000,   /* carveout starts at 448 */
+		.base		= 0x1C000000,   /* carveout starts at 448 MB */
 		.size		= SZ_64M - 0xC00000,
 		.buddy_size	= SZ_32K,
 	},
@@ -418,22 +340,13 @@ int __init paz00_panel_init(size_t fb_addr)
 {
 	int err;
 
-	gpio_request(TEGRA_EN_VDD_PNL, "en_vdd_pnl");
-	gpio_direction_output(TEGRA_EN_VDD_PNL, 1);
-	tegra_gpio_enable(TEGRA_EN_VDD_PNL);
-	gpio_free(TEGRA_EN_VDD_PNL);
+	gpio_request_one(TEGRA_EN_VDD_PNL, GPIOF_OUT_INIT_HIGH, "en_vdd_pnl");
+	gpio_request_one(TEGRA_BACKLIGHT_VDD, GPIOF_OUT_INIT_HIGH, "bl_vdd");
+	gpio_request_one(TEGRA_LVDS_SHUTDOWN, GPIOF_OUT_INIT_HIGH, "lvds_shdn");
+	gpio_request_one(TEGRA_HDMI_HPD, GPIOF_IN, "hdmi_hpd");
+	gpio_export(TEGRA_LVDS_SHUTDOWN, 0);
 
-	gpio_request(TEGRA_BACKLIGHT_VDD, "bl_vdd");
-	gpio_direction_output(TEGRA_BACKLIGHT_VDD, 1);
-	tegra_gpio_enable(TEGRA_BACKLIGHT_VDD);
-	gpio_free(TEGRA_BACKLIGHT_VDD);
-
-	gpio_request(TEGRA_HDMI_HPD, "hdmi_hpd");
-	gpio_direction_input(TEGRA_HDMI_HPD);
-	tegra_gpio_enable(TEGRA_HDMI_HPD);
-	gpio_free(TEGRA_HDMI_HPD);
-
-	paz00_disp1_resources[2].start = (unsigned long)fb_addr;
+	paz00_disp1_resources[2].start = fb_addr;
 
 	err = platform_add_devices(paz00_gfx_devices,
 				   ARRAY_SIZE(paz00_gfx_devices));
@@ -453,34 +366,34 @@ int __init paz00_panel_init(size_t fb_addr)
 
 static int __init paz00_hdmi_late_init(void)
 {
-        int ret;
+	int ret;
 
-        paz00_hdmi_reg = regulator_get(NULL, "avdd_hdmi");
-        if (IS_ERR_OR_NULL(paz00_hdmi_reg)) {
-                ret = PTR_ERR(paz00_hdmi_reg);
-                goto fail;
-        }
+	paz00_hdmi_reg = regulator_get(NULL, "avdd_hdmi");
+	if (IS_ERR_OR_NULL(paz00_hdmi_reg)) {
+		ret = PTR_ERR(paz00_hdmi_reg);
+		goto fail;
+	}
 
-        paz00_hdmi_pll = regulator_get(NULL, "avdd_hdmi_pll");
-        if (IS_ERR_OR_NULL(paz00_hdmi_pll)) {
-                ret = PTR_ERR(paz00_hdmi_pll);
-                goto fail;
-        }
+	paz00_hdmi_pll = regulator_get(NULL, "avdd_hdmi_pll");
+	if (IS_ERR_OR_NULL(paz00_hdmi_pll)) {
+		ret = PTR_ERR(paz00_hdmi_pll);
+		goto fail;
+	}
 
-        return 0;
+	return 0;
 
 fail:
-        if (paz00_hdmi_pll) {
-                regulator_disable(paz00_hdmi_pll);
-                paz00_hdmi_pll = NULL;
-        }
+	if (paz00_hdmi_pll) {
+		regulator_disable(paz00_hdmi_pll);
+		paz00_hdmi_pll = NULL;
+	}
 
-        if (paz00_hdmi_reg) {
-                regulator_disable(paz00_hdmi_reg);
-                paz00_hdmi_reg = NULL;
-        }
+	if (paz00_hdmi_reg) {
+		regulator_disable(paz00_hdmi_reg);
+		paz00_hdmi_reg = NULL;
+	}
 
-        return ret;
+	return ret;
 }
 
 late_initcall(paz00_hdmi_late_init);
