@@ -24,12 +24,12 @@
 #include <linux/clk.h>
 #include <linux/dma-mapping.h>
 #include <linux/gpio_keys.h>
+#include <linux/pda_power.h>
 #include <linux/i2c.h>
 #include <linux/i2c-tegra.h>
-#include <linux/io.h>
 #include <linux/input.h>
+#include <linux/io.h>
 #include <linux/rfkill-gpio.h>
-#include <linux/platform_data/tegra_usb.h>
 #include <linux/mfd/core.h>
 
 #include <asm/mach-types.h>
@@ -40,20 +40,21 @@
 #include <mach/iomap.h>
 #include <mach/irqs.h>
 #include <mach/sdhci.h>
-#include <mach/usb_phy.h>
 #include <mach/gpio.h>
-#include <mach/suspend.h>
-#include <mach/paz00_audio.h>
+#include <mach/pinmux.h>
+#include <mach/pinmux-t2.h>
+#include <mach/tegra_alc5632_pdata.h>
 
 #include "board.h"
 #include "board-paz00.h"
 #include "clock.h"
 #include "devices.h"
 #include "gpio-names.h"
+#include "pm.h"
 #include "../../../drivers/staging/nvec/nvec.h"
 
-#define ATAG_NVIDIA             0x41000801
-#define MAX_MEMHDL		8
+#define ATAG_NVIDIA	0x41000801
+#define MAX_MEMHDL	8
 
 struct tag_tegra {
 	__u32 bootarg_len;
@@ -142,14 +143,17 @@ __tagtable(ATAG_NVIDIA, parse_tag_nvidia);
 
 static struct plat_serial8250_port debug_uart_platform_data[] = {
 	{
+		/* serial port on JP1 */
 		.membase	= IO_ADDRESS(TEGRA_UARTA_BASE),
 		.mapbase	= TEGRA_UARTA_BASE,
 		.irq		= INT_UARTA,
-		.flags		= UPF_BOOT_AUTOCONF,
+		.flags		= UPF_BOOT_AUTOCONF | UPF_FIXED_TYPE,
+		.type		= PORT_TEGRA,
 		.iotype		= UPIO_MEM,
 		.regshift	= 2,
 		.uartclk	= 216000000,
 	}, {
+		/* serial port on mini-pcie */
 		.membase	= IO_ADDRESS(TEGRA_UARTD_BASE),
 		.mapbase	= TEGRA_UARTD_BASE,
 		.irq		= INT_UARTD,
@@ -171,41 +175,77 @@ static struct platform_device debug_uart = {
 	},
 };
 
-static struct tegra_i2c_platform_data paz00_i2c1_platform_data = {
-	.adapter_nr	= 0,
-	.bus_count	= 1,
-	.bus_clk_rate   = { 400000, 0 },
-	.slave_addr	= 0xfc,
+static struct rfkill_gpio_platform_data wifi_rfkill_platform_data = {
+	.name		= "wifi_rfkill",
+	.reset_gpio	= TEGRA_WIFI_RST,
+	.shutdown_gpio	= TEGRA_WIFI_PWRN,
+	.type	= RFKILL_TYPE_WLAN,
 };
 
-static const struct tegra_pingroup_config i2c2_ddc = {
-	.pingroup	= TEGRA_PINGROUP_DDC,
-	.func		= TEGRA_MUX_I2C2,
+static struct platform_device wifi_rfkill_device = {
+	.name	= "rfkill_gpio",
+	.id	= -1,
+	.dev	= {
+		.platform_data = &wifi_rfkill_platform_data,
+	},
 };
 
-static const struct tegra_pingroup_config i2c2_gen2 = {
-	.pingroup	= TEGRA_PINGROUP_PTA,
-	.func		= TEGRA_MUX_I2C2,
+static struct gpio_led gpio_leds[] = {
+	{
+		.name			= "wifi-led",
+		.default_trigger	= "rfkill0",
+		.gpio			= TEGRA_WIFI_LED,
+	},
 };
 
-static struct tegra_i2c_platform_data paz00_i2c2_platform_data = {
-	.adapter_nr	= 1,
-	.bus_count	= 2,
-	.bus_clk_rate   = { 100000, 100000 },
-	.bus_mux	= { &i2c2_ddc, &i2c2_gen2 },
-	.bus_mux_len	= { 1, 1 },
-	.slave_addr	= 0xfc,
+static struct gpio_led_platform_data gpio_led_info = {
+	.leds		= gpio_leds,
+	.num_leds	= ARRAY_SIZE(gpio_leds),
 };
 
-static struct tegra_i2c_platform_data paz00_i2c3_platform_data = {
-	.adapter_nr	= 3,
-	.bus_count	= 1,
-	.bus_clk_rate   = { 80000, 0 },
-	.is_slave	= true,
-	.slave_addr	= 0x8a,
+static struct platform_device leds_gpio = {
+	.name	= "leds-gpio",
+	.id	= -1,
+	.dev	= {
+		.platform_data = &gpio_led_info,
+	},
 };
 
-static const struct mfd_cell paz00_nvec_devices[] = {
+static struct gpio_keys_button paz00_gpio_keys_buttons[] = {
+	{
+		.code		= KEY_POWER,
+		.gpio		= TEGRA_GPIO_POWERKEY,
+		.active_low	= 1,
+		.desc		= "Power",
+		.type		= EV_KEY,
+		.wakeup		= 1,
+	},
+};
+
+static struct gpio_keys_platform_data paz00_gpio_keys = {
+	.buttons	= paz00_gpio_keys_buttons,
+	.nbuttons	= ARRAY_SIZE(paz00_gpio_keys_buttons),
+};
+
+static struct platform_device paz00_gpio_keys_device = {
+	.name	= "gpio-keys",
+	.id	= -1,
+	.dev	= {
+		.platform_data = &paz00_gpio_keys,
+	},
+};
+
+static struct tegra_suspend_platform_data paz00_suspend = {
+	.cpu_timer	= 5000,
+	.cpu_off_timer	= 5000,
+	.core_timer	= 0x7e7e,
+	.core_off_timer = 0x7f,
+	.corereq_high	= false,
+	.sysclkreq_high = true,
+	.suspend_mode	= TEGRA_SUSPEND_LP0,
+};
+
+static struct mfd_cell paz00_nvec_devices[] = {
 	{
 		.name = "nvec-kbd",
 		.id = 1,
@@ -248,13 +288,13 @@ static struct platform_device nvec_device = {
 	}
 };
 
-static struct paz00_audio_platform_data audio_pdata = {
+static struct tegra_alc5632_audio_platform_data audio_pdata = {
 /* speaker enable goes via nvec */
 	.gpio_hp_det	= TEGRA_HP_DET,
 };
 
 static struct platform_device audio_device = {
-	.name	= "tegra-snd-paz00",
+	.name	= "tegra-snd-alc5632",
 	.id	= 0,
 	.dev	= {
 		.platform_data = &audio_pdata,
@@ -265,18 +305,67 @@ static struct i2c_board_info __initdata alc5632_board_info = {
 	I2C_BOARD_INFO("alc5632", 0x1e),
 };
 
-static struct i2c_board_info __initdata atd7461_board_info = {
-	I2C_BOARD_INFO("lm90", 0x4c),
+static struct i2c_board_info __initdata adt7421_device = {
+	I2C_BOARD_INFO("adt7461", 0x4c),
+};
+
+static struct platform_device *paz00_devices[] __initdata = {
+	&debug_uart,
+	&tegra_pmu_device,
+	&tegra_gart_device,
+	&tegra_sdhci_device4,
+	&tegra_sdhci_device1,
+	&wifi_rfkill_device,
+	&paz00_gpio_keys_device,
+	&leds_gpio,
+	&nvec_device,
+	&tegra_i2s_device1,
+	&tegra_das_device,
+	&tegra_pcm_device,
+	&audio_device,
+	&tegra_avp_device,
+};
+
+static struct tegra_i2c_platform_data paz00_i2c1_platform_data = {
+	.adapter_nr	= 0,
+	.bus_count	= 1,
+	.bus_clk_rate	= { 400000, 0 },
+};
+
+static const struct tegra_pingroup_config i2c2_ddc = {
+	.pingroup	= TEGRA_PINGROUP_DDC,
+	.func	= TEGRA_MUX_I2C2,
+};
+
+static const struct tegra_pingroup_config i2c2_gen2 = {
+	.pingroup	= TEGRA_PINGROUP_PTA,
+	.func	= TEGRA_MUX_I2C2,
+};
+
+static struct tegra_i2c_platform_data paz00_i2c2_platform_data = {
+	.adapter_nr	= 1,
+	.bus_count	= 2,
+	.bus_clk_rate	= { 100000, 100000 },
+	.bus_mux	= { &i2c2_ddc, &i2c2_gen2 },
+	.bus_mux_len	= { 1, 1 },
+};
+
+static struct tegra_i2c_platform_data paz00_i2c3_platform_data = {
+	.adapter_nr	= 3,
+	.bus_count	= 1,
+	.bus_clk_rate	= { 80000, 0 },
+	.is_slave	= true,
+	.slave_addr	= 0x8a,
 };
 
 static struct tegra_i2c_platform_data paz00_dvc_platform_data = {
 	.adapter_nr	= 4,
 	.bus_count	= 1,
-	.bus_clk_rate   = { 400000, 0 },
+	.bus_clk_rate	= { 400000, 0 },
 	.is_dvc		= true,
 };
 
-static void paz00_i2c_init(void)
+static void __init paz00_i2c_init(void)
 {
 	tegra_i2c_device1.dev.platform_data = &paz00_i2c1_platform_data;
 	tegra_i2c_device2.dev.platform_data = &paz00_i2c2_platform_data;
@@ -289,38 +378,11 @@ static void paz00_i2c_init(void)
 	platform_device_register(&tegra_i2c_device4);
 
 	i2c_register_board_info(0, &alc5632_board_info, 1);
-	i2c_register_board_info(4, &atd7461_board_info, 1);
-
-	platform_device_register(&nvec_device);
+	i2c_register_board_info(4, &adt7421_device, 1);
 }
 
-static struct tegra_ulpi_config ulpi_phy_config = {
-		.reset_gpio = TEGRA_ULPI_RST,
-		.clk = "cdev2",
-};
-
-static struct tegra_ehci_platform_data tegra_ehci_pdata[] = {
-		[0] = {
-			.operating_mode = TEGRA_USB_HOST,
-			.power_down_on_bus_suspend = 1,
-		},
-		[1] = {
-			.phy_config = &ulpi_phy_config,
-			.operating_mode = TEGRA_USB_HOST,
-			.power_down_on_bus_suspend = 1,
-		},
-		[2] = {
-			.operating_mode = TEGRA_USB_HOST,
-			//don't power down phy to keep hotplug working
-			.power_down_on_bus_suspend = 0,
-		},
-};
-
 static void paz00_usb_init(void)
 {
-	tegra_ehci2_device.dev.platform_data = &tegra_ehci_pdata[1];
-	tegra_ehci3_device.dev.platform_data = &tegra_ehci_pdata[2];
-
 	platform_device_register(&tegra_ehci2_device);
 	platform_device_register(&tegra_ehci3_device);
 }
@@ -333,112 +395,22 @@ static void __init tegra_paz00_fixup(struct machine_desc *desc,
 	mi->bank[0].size = 448 * SZ_1M;
 }
 
-static struct gpio_led gpio_leds[] = {
-	{
-		.name			= "wifi-led",
-		.default_trigger	= "rfkill0",
-		.gpio			= TEGRA_WIFI_LED,
-	},
-};
-
-static struct gpio_led_platform_data gpio_led_info = {
-		.leds           = gpio_leds,
-		.num_leds       = ARRAY_SIZE(gpio_leds),
-};
-
-static struct platform_device leds_gpio = {
-	.name   = "leds-gpio",
-	.id     = -1,
-	.dev    = {
-		.platform_data  = &gpio_led_info,
-	},
-};
-
-static struct rfkill_gpio_platform_data wifi_rfkill_platform_data = {
-	.name           = "wifi_rfkill",
-	.reset_gpio     = TEGRA_WIFI_RST,
-	.shutdown_gpio  = TEGRA_WIFI_PWRN,
-	.type           = RFKILL_TYPE_WLAN
-};
-
-static struct platform_device wifi_rfkill_device = {
-	.name   = "rfkill_gpio",
-	.id     = -1,
-	.dev    = {
-		.platform_data = &wifi_rfkill_platform_data,
-	},
-};
-
-static struct gpio_keys_button paz00_gpio_keys_buttons[] = {
-	{
-		.code		= KEY_POWER,
-		.gpio		= TEGRA_GPIO_POWERKEY,
-		.active_low	= 1,
-		.desc		= "Power",
-		.type		= EV_KEY,
-		.wakeup		= 1,
-	},
-};
-
-static struct gpio_keys_platform_data paz00_gpio_keys = {
-	.buttons	= paz00_gpio_keys_buttons,
-	.nbuttons	= ARRAY_SIZE(paz00_gpio_keys_buttons),
-};
-
-static struct platform_device paz00_gpio_keys_device = {
-	.name	= "gpio-keys",
-	.id	= -1,
-	.dev	= {
-		.platform_data = &paz00_gpio_keys,
-	},
-};
-
-static struct platform_device *paz00_devices[] __initdata = {
-	&debug_uart,
-	&tegra_pmu_device,
-//	&tegra_rtc_device,
-	&tegra_udc_device,
-	&tegra_sdhci_device4,
-	&tegra_sdhci_device1,
-	&tegra_spi_device1,
-	&tegra_spi_device2,
-	&tegra_spi_device3,
-	&tegra_spi_device4,
-	&wifi_rfkill_device,
-	&leds_gpio,
-	&paz00_gpio_keys_device,
-	&tegra_gart_device,
-	&tegra_i2s_device1,
-	&tegra_das_device,
-	&tegra_pcm_device,
-	&audio_device,
-	&tegra_avp_device,
-};
-
 static __initdata struct tegra_clk_init_table paz00_clk_init_table[] = {
 	/* name		parent		rate		enabled */
-/* for serial port */
+	{ "uarta",	"pll_p",	216000000,	true  },
 	{ "uartd",	"pll_p",	216000000,	true  },
 
-/* these are switched on by bootloader */
-	{ "audio",	"pll_a_out0",	11289600,	true  },
-	{ "uarta",	"pll_p",	216000000,	true  },
-	{ "i2c3_i2c",	"pll_p_out3",	72000000,	true  },
-	{ "clk_d",	"clk_m",	24000000,	true  },
-	{ "pll_a_out0",	"pll_a",	11289600,	true  },
 	{ "pll_a",	"pll_p_out1",	56448000,	true  },
+	{ "pll_a_out0",	"pll_a",	11289600,	true  },
 	{ "pll_p_out4",	"pll_p",	24000000,	true  },
-	{ "blink",	"clk_32k",	32768,		true  }, 
-
-/* these are used for audio */
-	{ "cdev1",	"pll_a_out0",	11289600,	true  },
-	{ "audio_2x",	"audio",	22579200,	false },
-	{ "i2s1",	"pll_a_out0",	11289600,	false },
-
-/* these are used for usb */
+	{ "usbd",	"clk_m",	12000000,	false },
 	{ "usb2",	"clk_m",	12000000,	false },
 	{ "usb3",	"clk_m",	12000000,	false },
-	{ "usbd",	"clk_m",	12000000,	false },
+
+	/* these are used for audio */
+	{ "cdev1",	NULL,		0,		true  },
+	{ "audio_2x",	"audio",	22579200,	false },
+	{ "i2s1",	"pll_a_out0",	11289600,	false },
 
 	{ NULL,		NULL,		0,		0},
 };
@@ -456,17 +428,6 @@ static struct tegra_sdhci_platform_data sdhci_pdata4 = {
 	.is_8bit	= 1,
 };
 
-static struct tegra_suspend_platform_data paz00_suspend = {
-	.cpu_timer	= 5000,
-	.cpu_off_timer	= 5000,
-	.core_timer	= 0x7e7e,
-	.core_off_timer = 0x7f,
-	.separate_req	= true,
-	.corereq_high	= false,
-	.sysclkreq_high = true,
-	.suspend_mode	= TEGRA_SUSPEND_LP0,
-};
-
 static void __init tegra_paz00_init(void)
 {
 	tegra_init_suspend(&paz00_suspend);
@@ -478,13 +439,13 @@ static void __init tegra_paz00_init(void)
 	tegra_sdhci_device1.dev.platform_data = &sdhci_pdata1;
 	tegra_sdhci_device4.dev.platform_data = &sdhci_pdata4;
 
-	platform_add_devices(paz00_devices, ARRAY_SIZE(paz00_devices));
-
+	paz00_emc_init();
 	paz00_i2c_init();
-	paz00_power_init();
+	paz00_regulator_init();
 	paz00_panel_init(fb_addr);
 	paz00_usb_init();
-	paz00_emc_init();
+
+	platform_add_devices(paz00_devices, ARRAY_SIZE(paz00_devices));
 }
 
 MACHINE_START(PAZ00, "Toshiba AC100 / Dynabook AZ")
@@ -496,6 +457,3 @@ MACHINE_START(PAZ00, "Toshiba AC100 / Dynabook AZ")
 	.timer          = &tegra_timer,
 	.init_machine   = tegra_paz00_init,
 MACHINE_END
-
-MACHINE_START(HARMONY, "harmony")
-MACHINE_END
