@@ -19,36 +19,32 @@
 #include <linux/platform_device.h>
 #include <linux/serial_8250.h>
 #include <linux/i2c.h>
-#include <linux/i2c-tegra.h>
-#include <linux/i2c/atmel_mxt_ts.h>
-#include <linux/i2c/cyapa.h>
 #include <linux/delay.h>
 #include <linux/input.h>
 #include <linux/io.h>
 #include <linux/gpio.h>
 #include <linux/gpio_keys.h>
-#include <mach/usb_phy.h>
-#include <linux/platform_data/tegra_usb.h>
-#include <linux/nct1008.h>
+#include <linux/i2c-tegra.h>
+#include <linux/i2c/atmel_mxt_ts.h>
+#include <linux/i2c/cyapa.h>
+#include <linux/clk.h>
 #include <linux/power/bq20z75.h>
 #include <linux/rfkill-gpio.h>
-#include <linux/leds.h>
-#include <linux/leds_pwm.h>
+#include <linux/platform_data/tegra_usb.h>
 #include <linux/memblock.h>
 
-#include <sound/max98095.h>
 #include <sound/wm8903.h>
 
 #include <mach/iomap.h>
 #include <mach/irqs.h>
 #include <mach/sdhci.h>
+#include <mach/tegra_wm8903_pdata.h>
 #include <mach/pinmux.h>
 #include <mach/pinmux-t2.h>
 #include <mach/kbc.h>
-#include <mach/suspend.h>
 #include <mach/system.h>
-#include <mach/seaboard_audio.h>
 #include <mach/clk.h>
+#include <mach/usb_phy.h>
 
 #include <asm/cacheflush.h>
 #include <asm/mach-types.h>
@@ -59,6 +55,7 @@
 #include "clock.h"
 #include "devices.h"
 #include "gpio-names.h"
+#include "pm.h"
 
 extern void tegra_throttling_enable(bool);
 
@@ -87,14 +84,15 @@ static struct platform_device debug_uart = {
 
 static __initdata struct tegra_clk_init_table seaboard_clk_init_table[] = {
 	/* name		parent		rate		enabled */
-	{ "pll_p_out4", "pll_p",        24000000,       true},
+	{ "pll_p_out4",	"pll_p",	24000000,	true},
 	{ "pll_a",      "pll_p_out1",   56448000,       true},
 	{ "pll_a_out0", "pll_a",        11289600,       true},
-	{ "cdev1",      "pll_a_out0",   11289600,       true},
+	{ "cdev1",      NULL,           0,              true},
 	{ "i2s1",       "pll_a_out0",   11289600,       false},
 	{ "audio",      "pll_a_out0",   11289600,       false},
 	{ "audio_2x",   "audio",        22579200,       false},
 	{ "spdif_out",  "pll_a_out0",   11289600,       false},
+	{ "vi_sensor",  "pll_m",        24000000,       false},
 	{ "uartb",      "pll_p",        216000000,      false},
 	{ "uartc",      "pll_c",        600000000,      false},
 	{ "uartd",      "pll_p",        216000000,      false},
@@ -103,94 +101,41 @@ static __initdata struct tegra_clk_init_table seaboard_clk_init_table[] = {
 	{ NULL,		NULL,		0,		0},
 };
 
-static struct tegra_utmip_config utmi_phy_config[] = {
-	[0] = {
-		.hssync_start_delay = 0,
-		.idle_wait_delay = 17,
-		.elastic_limit = 16,
-		.term_range_adj = 6,
-		.xcvr_setup = 15,
-		.xcvr_lsfslew = 2,
-		.xcvr_lsrslew = 2,
-		.vbus_gpio = TEGRA_GPIO_USB1,
-	},
-	[1] = {
-		.hssync_start_delay = 0,
-		.idle_wait_delay = 17,
-		.elastic_limit = 16,
-		.term_range_adj = 6,
-		.xcvr_setup = 8,
-		.xcvr_lsfslew = 2,
-		.xcvr_lsrslew = 2,
-		.vbus_gpio = TEGRA_GPIO_USB3,
-		.shared_pin_vbus_en_oc = true,
-	},
-};
-
-static struct tegra_ulpi_config ulpi_phy_config = {
-	.reset_gpio = TEGRA_GPIO_PV1,
-	.clk = "cdev2",
-};
-
-static struct tegra_ehci_platform_data tegra_ehci_pdata[] = {
-	[0] = {
-		.phy_config = &utmi_phy_config[0],
-		.operating_mode = TEGRA_USB_HOST,
-		.power_down_on_bus_suspend = 1,
-		.keep_clock_in_bus_suspend = 1,
-	},
-	[1] = {
-		.phy_config = &ulpi_phy_config,
-		.operating_mode = TEGRA_USB_HOST,
-		.power_down_on_bus_suspend = 1,
-		.keep_clock_in_bus_suspend = 1,
-	},
-	[2] = {
-		.phy_config = &utmi_phy_config[1],
-		.operating_mode = TEGRA_USB_HOST,
-		.power_down_on_bus_suspend = 1,
-		.keep_clock_in_bus_suspend = 1,
-	},
-};
-
 static struct tegra_i2c_platform_data seaboard_i2c1_platform_data = {
-        .adapter_nr     = 0,
-        .bus_count      = 1,
-        .bus_clk_rate   = { 400000, 0 },
-	.slave_addr	= 0xfc,
+	.adapter_nr	= 0,
+	.bus_count	= 1,
+	.bus_clk_rate	= { 400000, 0 },
 };
 
 static const struct tegra_pingroup_config i2c2_ddc = {
-        .pingroup       = TEGRA_PINGROUP_DDC,
-        .func           = TEGRA_MUX_I2C2,
+	.pingroup	= TEGRA_PINGROUP_DDC,
+	.func		= TEGRA_MUX_I2C2,
 };
 
 static const struct tegra_pingroup_config i2c2_gen2 = {
-        .pingroup       = TEGRA_PINGROUP_PTA,
-        .func           = TEGRA_MUX_I2C2,
+	.pingroup	= TEGRA_PINGROUP_PTA,
+	.func		= TEGRA_MUX_I2C2,
 };
 
 static struct tegra_i2c_platform_data seaboard_i2c2_platform_data = {
-        .adapter_nr     = 1,
-        .bus_count      = 2,
-        .bus_clk_rate   = { 100000, 100000 },
-        .bus_mux        = { &i2c2_ddc, &i2c2_gen2 },
-        .bus_mux_len    = { 1, 1 },
-	.slave_addr	= 0xfc,
+	.adapter_nr	= 1,
+	.bus_count	= 2,
+	.bus_clk_rate	= { 100000, 100000 },
+	.bus_mux	= { &i2c2_ddc, &i2c2_gen2 },
+	.bus_mux_len	= { 1, 1 },
 };
 
 static struct tegra_i2c_platform_data seaboard_i2c3_platform_data = {
-        .adapter_nr     = 3,
-        .bus_count      = 1,
-        .bus_clk_rate   = { 400000, 0 },
-	.slave_addr	= 0xfc,
+	.adapter_nr	= 3,
+	.bus_count	= 1,
+	.bus_clk_rate	= { 400000, 0 },
 };
 
 static struct tegra_i2c_platform_data seaboard_dvc_platform_data = {
-        .adapter_nr     = 4,
-        .bus_count      = 1,
-        .bus_clk_rate   = { 400000, 0 },
-        .is_dvc         = true,
+	.adapter_nr	= 4,
+	.bus_count	= 1,
+	.bus_clk_rate	= { 400000, 0 },
+	.is_dvc		= true,
 };
 
 static struct gpio_keys_button seaboard_gpio_keys_buttons[] = {
@@ -322,8 +267,8 @@ static const u32 cros_kbd_keymap[] = {
 };
 
 static const struct matrix_keymap_data cros_keymap_data = {
-        .keymap         = cros_kbd_keymap,
-        .keymap_size    = ARRAY_SIZE(cros_kbd_keymap),
+	.keymap		= cros_kbd_keymap,
+	.keymap_size	= ARRAY_SIZE(cros_kbd_keymap),
 };
 
 static struct tegra_kbc_platform_data seaboard_kbc_platform_data = {
@@ -335,26 +280,26 @@ static struct tegra_kbc_platform_data seaboard_kbc_platform_data = {
 
 static void seaboard_kbc_init(void)
 {
-       struct tegra_kbc_platform_data *data = &seaboard_kbc_platform_data;
-       int i, j;
+	struct tegra_kbc_platform_data *data = &seaboard_kbc_platform_data;
+	int i, j;
 
-       BUG_ON((KBC_MAX_ROW + KBC_MAX_COL) > KBC_MAX_GPIO);
-       /*
-        * Setup the pin configuration information.
-        */
-       for (i = 0; i < KBC_MAX_ROW; i++) {
-               data->pin_cfg[i].num = i;
-               data->pin_cfg[i].is_row = true;
-       }
+	BUG_ON((KBC_MAX_ROW + KBC_MAX_COL) > KBC_MAX_GPIO);
+	/*
+	 * Setup the pin configuration information.
+	 */
+	for (i = 0; i < KBC_MAX_ROW; i++) {
+		data->pin_cfg[i].num = i;
+		data->pin_cfg[i].is_row = true;
+	}
 
-       for (j = 0; j < KBC_MAX_COL; j++) {
-               data->pin_cfg[i + j].num = j;
-               data->pin_cfg[i + j].is_row = false;
-       }
+	for (j = 0; j < KBC_MAX_COL; j++) {
+		data->pin_cfg[i + j].num = j;
+		data->pin_cfg[i + j].is_row = false;
+	}
 
-       tegra_kbc_device.dev.platform_data = data;
+	tegra_kbc_device.dev.platform_data = data;
 
-       platform_device_register(&tegra_kbc_device);
+	platform_device_register(&tegra_kbc_device);
 }
 
 static struct rfkill_gpio_platform_data bt_rfkill_platform_data = {
@@ -392,17 +337,20 @@ static struct tegra_sdhci_platform_data sdhci_pdata4 = {
 	.is_8bit	= 1,
 };
 
-static struct seaboard_audio_platform_data seaboard_audio_pdata = {
-	.gpio_spkr_en = TEGRA_GPIO_SPKR_EN,
-	.gpio_hp_det = TEGRA_GPIO_HP_DET,
-	.gpio_hp_mute = -1,
+static struct tegra_wm8903_platform_data seaboard_audio_pdata = {
+	.gpio_spkr_en		= TEGRA_GPIO_SPKR_EN,
+	.gpio_hp_det		= TEGRA_GPIO_HP_DET,
+	.gpio_hp_mute		= -1,
+	.gpio_int_mic_en	= -1,
+	.gpio_ext_mic_en	= -1,
+	.gpio_hp_invert		= 1,
 };
 
 static struct platform_device seaboard_audio_device = {
-	.name = "tegra-snd-seaboard", /* must match DRV_NAME in seaboard.c */
-	.id   = 0,
-	.dev  = {
-		.platform_data = &seaboard_audio_pdata,
+	.name	= "tegra-snd-wm8903",
+	.id	= 0,
+	.dev	= {
+		.platform_data  = &seaboard_audio_pdata,
 	},
 };
 
@@ -411,28 +359,6 @@ static struct platform_device spdif_dit_device = {
 	.id     = -1,
 };
 
-static struct led_pwm arthur_pwm_leds[] = {
-	{
-		.name		= "tegra::kbd_backlight",
-		.pwm_id		= 1,
-		.max_brightness	= 255,
-		.pwm_period_ns	= 1000000,
-	},
-};
-
-static struct led_pwm_platform_data arthur_pwm_data = {
-	.leds		= arthur_pwm_leds,
-	.num_leds	= ARRAY_SIZE(arthur_pwm_leds),
-};
-
-static struct platform_device arthur_leds_pwm = {
-	.name	= "leds_pwm",
-	.id	= -1,
-	.dev	= {
-		.platform_data = &arthur_pwm_data,
-	},
-};
-
 static struct platform_device *seaboard_devices[] __initdata = {
 	&debug_uart,
 	&tegra_uartc_device,
@@ -443,31 +369,28 @@ static struct platform_device *seaboard_devices[] __initdata = {
 	&tegra_sdhci_device3,
 	&tegra_sdhci_device1,
 	&seaboard_gpio_keys_device,
-	&tegra_avp_device,
 	&tegra_i2s_device1,
 	&tegra_das_device,
 	&tegra_pcm_device,
+	&seaboard_audio_device,
 	&tegra_spdif_device,
 	&spdif_dit_device,
 	&bt_rfkill_device,
-	&seaboard_audio_device,
+	&tegra_avp_device,
 };
 
-static struct platform_device *arthur_specific_devices[] __initdata = {
-	&tegra_pwfm1_device,
-	&arthur_leds_pwm,
+static struct i2c_board_info __initdata isl29018_device = {
+	I2C_BOARD_INFO("isl29018", 0x44),
+	.irq = TEGRA_GPIO_TO_IRQ(TEGRA_GPIO_ISL29018_IRQ),
 };
 
-static struct nct1008_platform_data nct1008_pdata = {
-	.supported_hwrev	= true,
-	.ext_range		= false,
-	.conv_rate		= 0x08,
-	.offset			= 0,
-	.hysteresis		= 0,
-	.shutdown_ext_limit	= 115,
-	.shutdown_local_limit	= 120,
-	.throttling_ext_limit	= 90,
-	.alarm_fn		= tegra_throttling_enable,
+static struct i2c_board_info __initdata tsl2563_device = {
+	I2C_BOARD_INFO("tsl2563", 0x29),
+	.irq = TEGRA_GPIO_TO_IRQ(ASYMPTOTE_GPIO_TSL2563_IRQ),
+};
+
+static struct i2c_board_info __initdata adt7461_device = {
+	I2C_BOARD_INFO("adt7461", 0x4c),
 };
 
 static struct wm8903_platform_data wm8903_pdata = {
@@ -485,131 +408,10 @@ static struct wm8903_platform_data wm8903_pdata = {
 	},
 };
 
-static struct max98095_eq_cfg max98095_eq_cfg[] = {
-	{ /* Flat response */
-		.name = "FLAT",
-		.rate = 44100,
-		.band1 = { 0x2000, 0xC002, 0x4000, 0x00E9, 0x0000 },
-		.band2 = { 0x2000, 0xC00F, 0x4000, 0x02BC, 0x0000 },
-		.band3 = { 0x2000, 0xC0A7, 0x4000, 0x0916, 0x0000 },
-		.band4 = { 0x2000, 0xC5C2, 0x4000, 0x1A87, 0x0000 },
-		.band5 = { 0x2000, 0xF6B0, 0x4000, 0x3F51, 0x0000 },
-	},
-	{ /* Low pass Fc=1KHz */
-		.name = "LOWPASS1K",
-		.rate = 44100,
-		.band1 = { 0x205D, 0xC001, 0x3FEF, 0x002E, 0x02E0 },
-		.band2 = { 0x5B9A, 0xC093, 0x3AB2, 0x088B, 0x1981 },
-		.band3 = { 0x0D22, 0xC170, 0x26EA, 0x0D79, 0x32CF },
-		.band4 = { 0x0894, 0xC612, 0x01B3, 0x1B34, 0x3FFA },
-		.band5 = { 0x0815, 0x3FFF, 0xCF78, 0x0000, 0x29B7 },
-	},
-	{ /* BASS=-12dB, TREBLE=+9dB, Fc=5KHz */
-		.name = "HIBOOST",
-		.rate = 44100,
-		.band1 = { 0x0815, 0xC001, 0x3AA4, 0x0003, 0x19A2 },
-		.band2 = { 0x0815, 0xC103, 0x092F, 0x0B55, 0x3F56 },
-		.band3 = { 0x0E0A, 0xC306, 0x1E5C, 0x136E, 0x3856 },
-		.band4 = { 0x2459, 0xF665, 0x0CAA, 0x3F46, 0x3EBB },
-		.band5 = { 0x5BBB, 0x3FFF, 0xCEB0, 0x0000, 0x28CA },
-	},
-	{ /* BASS=12dB, TREBLE=+12dB */
-		.name = "LOUD12DB",
-		.rate = 44100,
-		.band1 = { 0x7FC1, 0xC001, 0x3EE8, 0x0020, 0x0BC7 },
-		.band2 = { 0x51E9, 0xC016, 0x3C7C, 0x033F, 0x14E9 },
-		.band3 = { 0x1745, 0xC12C, 0x1680, 0x0C2F, 0x3BE9 },
-		.band4 = { 0x4536, 0xD7E2, 0x0ED4, 0x31DD, 0x3E42 },
-		.band5 = { 0x7FEF, 0x3FFF, 0x0BAB, 0x0000, 0x3EED },
-	},
-	{
-		.name = "FLAT",
-		.rate = 16000,
-		.band1 = { 0x2000, 0xC004, 0x4000, 0x0141, 0x0000 },
-		.band2 = { 0x2000, 0xC033, 0x4000, 0x0505, 0x0000 },
-		.band3 = { 0x2000, 0xC268, 0x4000, 0x115F, 0x0000 },
-		.band4 = { 0x2000, 0xDA62, 0x4000, 0x33C6, 0x0000 },
-		.band5 = { 0x2000, 0x4000, 0x4000, 0x0000, 0x0000 },
-	},
-	{
-		.name = "LOWPASS1K",
-		.rate = 16000,
-		.band1 = { 0x2000, 0xC004, 0x4000, 0x0141, 0x0000 },
-		.band2 = { 0x5BE8, 0xC3E0, 0x3307, 0x15ED, 0x26A0 },
-		.band3 = { 0x0F71, 0xD15A, 0x08B3, 0x2BD0, 0x3F67 },
-		.band4 = { 0x0815, 0x3FFF, 0xCF78, 0x0000, 0x29B7 },
-		.band5 = { 0x0815, 0x3FFF, 0xCF78, 0x0000, 0x29B7 },
-	},
-	{ /* BASS=-12dB, TREBLE=+9dB, Fc=2KHz */
-		.name = "HIBOOST",
-		.rate = 16000,
-		.band1 = { 0x0815, 0xC001, 0x3BD2, 0x0009, 0x16BF },
-		.band2 = { 0x080E, 0xC17E, 0xF653, 0x0DBD, 0x3F43 },
-		.band3 = { 0x0F80, 0xDF45, 0xEE33, 0x36FE, 0x3D79 },
-		.band4 = { 0x590B, 0x3FF0, 0xE882, 0x02BD, 0x3B87 },
-		.band5 = { 0x4C87, 0xF3D0, 0x063F, 0x3ED4, 0x3FB1 },
-	},
-	{ /* BASS=12dB, TREBLE=+12dB */
-		.name = "LOUD12DB",
-		.rate = 16000,
-		.band1 = { 0x7FC1, 0xC001, 0x3D07, 0x0058, 0x1344 },
-		.band2 = { 0x2DA6, 0xC013, 0x3CF1, 0x02FF, 0x138B },
-		.band3 = { 0x18F1, 0xC08E, 0x244D, 0x0863, 0x34B5 },
-		.band4 = { 0x2BE0, 0xF385, 0x04FD, 0x3EC5, 0x3FCE },
-		.band5 = { 0x7FEF, 0x4000, 0x0BAB, 0x0000, 0x3EED },
-	},
-};
-
-static struct max98095_biquad_cfg max98095_bq_cfg[] = {
-	{
-		.name = "LP4K",
-		.rate = 44100,
-		.band1 = { 0x5019, 0xe0de, 0x03c2, 0x0784, 0x03c2 },
-		.band2 = { 0x5013, 0xe0e5, 0x03c1, 0x0783, 0x03c1 },
-	},
-	{
-		.name = "HP4K",
-		.rate = 44100,
-		.band1 = { 0x5019, 0xe0de, 0x2e4b, 0xa36a, 0x2e4b },
-		.band2 = { 0x5013, 0xe0e5, 0x2e47, 0xa371, 0x2e47 },
-	},
-};
-
-static struct max98095_pdata max98095_pdata = {
-	.eq_cfg		   = max98095_eq_cfg,
-	.eq_cfgcnt	   = ARRAY_SIZE(max98095_eq_cfg),
-	.bq_cfg		   = max98095_bq_cfg,
-	.bq_cfgcnt	   = ARRAY_SIZE(max98095_bq_cfg),
-	.digmic_left_mode  = 0, /* 0 -> analog, 1 -> digital mic */
-	.digmic_right_mode = 0, /* 0 -> analog, 1 -> digital mic */
-};
-
-static struct i2c_board_info __initdata max98095_device = {
-	I2C_BOARD_INFO("max98095", 0x10),
-	.platform_data = &max98095_pdata,
-	.irq = TEGRA_GPIO_TO_IRQ(TEGRA_GPIO_MAX98095_IRQ),
-};
-
 static struct i2c_board_info __initdata wm8903_device = {
 	I2C_BOARD_INFO("wm8903", 0x1a),
 	.platform_data = &wm8903_pdata,
-	.irq = TEGRA_GPIO_TO_IRQ(TEGRA_GPIO_WM8903_IRQ),
-};
-
-static struct i2c_board_info __initdata isl29018_device = {
-	I2C_BOARD_INFO("isl29018", 0x44),
-	.irq = TEGRA_GPIO_TO_IRQ(TEGRA_GPIO_ISL29018_IRQ),
-};
-
-static struct i2c_board_info __initdata tsl2563_device = {
-	I2C_BOARD_INFO("tsl2563", 0x29),
-	.irq = TEGRA_GPIO_TO_IRQ(ASYMPTOTE_GPIO_TSL2563_IRQ),
-};
-
-static struct i2c_board_info __initdata nct1008_device = {
-	I2C_BOARD_INFO("nct1008", 0x4c),
-	.irq = TEGRA_GPIO_TO_IRQ(TEGRA_GPIO_NCT1008_THERM2_IRQ),
-	.platform_data = &nct1008_pdata,
+	.irq = TEGRA_GPIO_TO_IRQ(TEGRA_GPIO_CDC_IRQ),
 };
 
 static struct bq20z75_platform_data bq20z75_pdata = {
@@ -628,11 +430,6 @@ static struct i2c_board_info __initdata ak8975_device = {
 	.irq		= TEGRA_GPIO_TO_IRQ(TEGRA_GPIO_MAGNETOMETER),
 };
 
-static struct i2c_board_info __initdata cyapa_device = {
-	I2C_BOARD_INFO(CYAPA_I2C_NAME, 0x67),
-	.irq		= TEGRA_GPIO_TO_IRQ(TEGRA_GPIO_CYTP_INT),
-};
-
 static struct i2c_board_info __initdata mpu3050_device = {
 	I2C_BOARD_INFO("mpu3050", 0x68),
 	.irq            = TEGRA_GPIO_TO_IRQ(TEGRA_GPIO_MPU3050_IRQ),
@@ -765,28 +562,63 @@ static __initdata struct tegra_pingroup_config mxt_pinmux_config[] = {
 	{TEGRA_PINGROUP_LVP0,  TEGRA_MUX_RSVD4,         TEGRA_PUPD_NORMAL,    TEGRA_TRI_NORMAL},
 };
 
-static void register_ehci_device(struct platform_device *pdev)
-{
-	if (pdev->dev.platform_data)
-		platform_device_register(pdev);
-}
+static struct i2c_board_info __initdata cyapa_device = {
+	I2C_BOARD_INFO(CYAPA_I2C_NAME, 0x67),
+	.irq		= TEGRA_GPIO_TO_IRQ(TEGRA_GPIO_CYTP_INT),
+};
+
+static struct tegra_utmip_config usb1_phy_config = {
+	.hssync_start_delay = 0,
+	.idle_wait_delay = 17,
+	.elastic_limit = 16,
+	.term_range_adj = 6,
+	.xcvr_setup = 15,
+	.xcvr_lsfslew = 2,
+	.xcvr_lsrslew = 2,
+	.vbus_gpio = TEGRA_GPIO_USB1,
+};
+
+static struct tegra_utmip_config usb3_phy_config = {
+	.hssync_start_delay = 0,
+	.idle_wait_delay = 17,
+	.elastic_limit = 16,
+	.term_range_adj = 6,
+	.xcvr_setup = 8,
+	.xcvr_lsfslew = 2,
+	.xcvr_lsrslew = 2,
+	.vbus_gpio = TEGRA_GPIO_USB3,
+	.shared_pin_vbus_en_oc = true,
+};
+
 
 static int seaboard_ehci_init(void)
 {
-	/* If we ever have a derivative that doesn't use USB1, make the code
-	 * below conditional
-	 */
-	BUG_ON(!tegra_ehci1_device.dev.platform_data);
+	int gpio_status;
+	struct tegra_ehci_platform_data *pdata;
 
-	if (utmi_phy_config[0].vbus_gpio)
-		tegra_usb_phy_utmi_vbus_init(&utmi_phy_config[0], "VBUS_USB1");
+	gpio_status = gpio_request(TEGRA_GPIO_USB1, "VBUS_USB1");
+	if (gpio_status < 0) {
+		pr_err("VBUS_USB1 request GPIO FAILED\n");
+		WARN_ON(1);
+	}
 
-	if (utmi_phy_config[1].vbus_gpio)
-		tegra_usb_phy_utmi_vbus_init(&utmi_phy_config[1], "VBUS_USB3");
+	gpio_status = gpio_direction_output(TEGRA_GPIO_USB1, 1);
+	if (gpio_status < 0) {
+		pr_err("VBUS_USB1 request GPIO DIRECTION FAILED\n");
+		WARN_ON(1);
+	}
+	gpio_set_value(TEGRA_GPIO_USB1, 1);
 
-	register_ehci_device(&tegra_ehci1_device);
-	register_ehci_device(&tegra_ehci2_device);
-	register_ehci_device(&tegra_ehci3_device);
+	pdata = tegra_ehci1_device.dev.platform_data;
+	pdata->keep_clock_in_bus_suspend = 1;
+	pdata->phy_config = &usb1_phy_config;
+
+	pdata = tegra_ehci3_device.dev.platform_data;
+	pdata->keep_clock_in_bus_suspend = 1;
+	pdata->phy_config = &usb3_phy_config;
+
+	platform_device_register(&tegra_ehci1_device);
+	platform_device_register(&tegra_ehci3_device);
 
 	return 0;
 }
@@ -834,7 +666,7 @@ static void __init seaboard_i2c_register_devices(void)
 
 	i2c_register_board_info(2, &bq20z75_device, 1);
 
-	i2c_register_board_info(4, &nct1008_device, 1);
+	i2c_register_board_info(4, &adt7461_device, 1);
 	i2c_register_board_info(4, &ak8975_device, 1);
 }
 
@@ -854,12 +686,12 @@ static void __init kaen_i2c_register_devices(void)
 
 	i2c_register_board_info(0, &wm8903_device, 1);
 	i2c_register_board_info(0, &isl29018_device, 1);
-	i2c_register_board_info(0, &cyapa_device, 1);
 	i2c_register_board_info(0, &mpu3050_device, 1);
+	i2c_register_board_info(0, &cyapa_device, 1);
 
 	i2c_register_board_info(2, &bq20z75_device, 1);
 
-	i2c_register_board_info(4, &nct1008_device, 1);
+	i2c_register_board_info(4, &adt7461_device, 1);
 	i2c_register_board_info(4, &ak8975_device, 1);
 }
 
@@ -879,12 +711,12 @@ static void __init wario_i2c_register_devices(void)
 
 	i2c_register_board_info(0, &wm8903_device, 1);
 	i2c_register_board_info(0, &isl29018_device, 1);
-	i2c_register_board_info(0, &cyapa_device, 1);
 	i2c_register_board_info(0, &mpu3050_device, 1);
+	i2c_register_board_info(0, &cyapa_device, 1);
 
 	i2c_register_board_info(2, &bq20z75_device, 1);
 
-	i2c_register_board_info(4, &nct1008_device, 1);
+	i2c_register_board_info(4, &adt7461_device, 1);
 	i2c_register_board_info(4, &ak8975_device, 1);
 }
 
@@ -904,12 +736,12 @@ static void __init aebl_i2c_register_devices(void)
 
 	i2c_register_board_info(0, &wm8903_device, 1);
 	i2c_register_board_info(0, &isl29018_device, 1);
-	i2c_register_board_info(0, &cyapa_device, 1);
 	i2c_register_board_info(0, &mpu3050_device, 1);
+	i2c_register_board_info(0, &cyapa_device, 1);
 
 	i2c_register_board_info(2, &bq20z75_device, 1);
 
-	i2c_register_board_info(4, &nct1008_device, 1);
+	i2c_register_board_info(4, &adt7461_device, 1);
 	i2c_register_board_info(4, &ak8975_device, 1);
 }
 
@@ -921,16 +753,9 @@ static void __init arthur_i2c_register_devices(void)
 	gpio_request(TEGRA_GPIO_NCT1008_THERM2_IRQ, "temp_alert");
 	gpio_direction_input(TEGRA_GPIO_NCT1008_THERM2_IRQ);
 
-	gpio_request(TEGRA_GPIO_CYTP_INT, "gpio_cytp_int");
-	gpio_direction_input(TEGRA_GPIO_CYTP_INT);
+	i2c_register_board_info(0, &isl29018_device, 1);
 
-	i2c_register_board_info(0, &cyapa_device, 1);
-
-	i2c_register_board_info(2, &bq20z75_device, 1);
-
-	i2c_register_board_info(3, &isl29018_device, 1);
-	i2c_register_board_info(0, &max98095_device, 1);
-	i2c_register_board_info(4, &nct1008_device, 1);
+	i2c_register_board_info(4, &adt7461_device, 1);
 }
 
 static void __init asymptote_i2c_register_devices(void)
@@ -966,11 +791,10 @@ static void __init asymptote_i2c_register_devices(void)
 	i2c_register_board_info(0, &tsl2563_device, 1);
 	i2c_register_board_info(2, &bq20z75_device, 1);
 	i2c_register_board_info(3, &asymptote_mxt_device, 1);
-	i2c_register_board_info(4, &nct1008_device, 1);
+	i2c_register_board_info(4, &adt7461_device, 1);
 }
 
-static void __init __seaboard_common_init(struct platform_device **devices,
-					  int num_devices)
+static void __init __seaboard_common_init(void)
 {
 	tegra_clk_init_from_table(seaboard_clk_init_table);
 
@@ -983,11 +807,10 @@ static void __init __seaboard_common_init(struct platform_device **devices,
 	tegra_sdhci_device4.dev.platform_data = &sdhci_pdata4;
 
 	platform_add_devices(seaboard_devices, ARRAY_SIZE(seaboard_devices));
-	if (devices)		/* Add board-specific devices. */
-		platform_add_devices(devices, num_devices);
 
 	seaboard_power_init();
 	seaboard_ehci_init();
+	seaboard_panel_init();
 	seaboard_kbc_init();
 
 	gpio_request(TEGRA_GPIO_RECOVERY_SWITCH, "recovery_switch");
@@ -1016,38 +839,31 @@ static void __init tegra_set_clock_readskew(const char *clk_name, int skew)
 static void __init seaboard_common_init(void)
 {
 	seaboard_pinmux_init();
-	__seaboard_common_init(NULL, 0);
+	__seaboard_common_init();
 }
 
 static void __init kaen_common_init(void)
 {
 	kaen_pinmux_init();
-	__seaboard_common_init(NULL, 0);
+	__seaboard_common_init();
 }
 
 static void __init aebl_common_init(void)
 {
 	aebl_pinmux_init();
-	__seaboard_common_init(NULL, 0);
-}
-
-static void __init arthur_common_init(void)
-{
-	arthur_pinmux_init();
-	__seaboard_common_init(arthur_specific_devices,
-			       ARRAY_SIZE(arthur_specific_devices));
+	__seaboard_common_init();
 }
 
 static void __init ventana_common_init(void)
 {
 	ventana_pinmux_init();
-	__seaboard_common_init(NULL, 0);
+	__seaboard_common_init();
 }
 
 static void __init asymptote_common_init(void)
 {
 	asymptote_pinmux_init();
-	__seaboard_common_init(NULL, 0);
+	__seaboard_common_init();
 }
 
 static struct tegra_suspend_platform_data seaboard_suspend = {
@@ -1055,7 +871,6 @@ static struct tegra_suspend_platform_data seaboard_suspend = {
 	.cpu_off_timer = 5000,
 	.core_timer = 0x7e7e,
 	.core_off_timer = 0x7f,
-	.separate_req = true,
 	.corereq_high = false,
 	.sysclkreq_high = true,
 	.suspend_mode = TEGRA_SUSPEND_LP0,
@@ -1091,12 +906,8 @@ static void __init tegra_seaboard_init(void)
 
 	__init_debug_uart_D();
 
-	tegra_ehci1_device.dev.platform_data = &tegra_ehci_pdata[0];
-	tegra_ehci2_device.dev.platform_data = &tegra_ehci_pdata[1];
-	tegra_ehci3_device.dev.platform_data = &tegra_ehci_pdata[2];
-
 	seaboard_common_init();
-	seaboard_panel_init();
+
 	seaboard_emc_init();
 
 	seaboard_i2c_register_devices();
@@ -1164,22 +975,16 @@ static void __init tegra_kaen_init(void)
 	 * diagram test.
 	 * xcvr_effect is only for USB1 to set FUSE_SETUP_SEL to zero
 	 */
-	utmi_phy_config[0].xcvr_effect = 1;
-	utmi_phy_config[0].xcvr_setup = 13;
-	utmi_phy_config[1].xcvr_setup = 13;
-
-	tegra_ehci1_device.dev.platform_data = &tegra_ehci_pdata[0];
-	tegra_ehci2_device.dev.platform_data = &tegra_ehci_pdata[1];
-	tegra_ehci3_device.dev.platform_data = &tegra_ehci_pdata[2];
+	usb1_phy_config.xcvr_effect = 1;
+	usb1_phy_config.xcvr_setup = 13;
+	usb3_phy_config.xcvr_setup = 13;
 
 	kaen_common_init();
-	kaen_panel_init();
 	kaen_emc_init();
 
 	kaen_i2c_register_devices();
 	seaboard_i2c_init();
 
-	kaen_sensors_init();
 	legacy_arm_pm_restart = arm_pm_restart;
 	arm_pm_restart = gpio_machine_restart;
 }
@@ -1205,17 +1010,14 @@ static void __init tegra_aebl_init(void)
 	/* setting skew makes WIFI stable when sdmmc1 runs 48MHz. */
 	tegra_set_clock_readskew("sdmmc1", 8);
 
-	tegra_ehci1_device.dev.platform_data = &tegra_ehci_pdata[0];
-	tegra_ehci2_device.dev.platform_data = &tegra_ehci_pdata[1];
-	tegra_ehci3_device.dev.platform_data = &tegra_ehci_pdata[2];
-
 	aebl_common_init();
-	seaboard_panel_init();
 	aebl_emc_init();
 
 	aebl_i2c_register_devices();
 	seaboard_i2c_init();
 
+	kaen_sensors_init();
+
 	aebl_sensors_init();
 	legacy_arm_pm_restart = arm_pm_restart;
 	arm_pm_restart = gpio_machine_restart;
@@ -1231,12 +1033,8 @@ static void __init tegra_wario_init(void)
 
 	seaboard_kbc_platform_data.keymap_data = &cros_keymap_data;
 
-	tegra_ehci1_device.dev.platform_data = &tegra_ehci_pdata[0];
-	tegra_ehci2_device.dev.platform_data = &tegra_ehci_pdata[1];
-	tegra_ehci3_device.dev.platform_data = &tegra_ehci_pdata[2];
-
 	seaboard_common_init();
-	wario_panel_init();
+
 	/* wario has same memory config as seaboard */
 	seaboard_emc_init();
 
@@ -1255,43 +1053,13 @@ static void __init tegra_wario_init(void)
 
 static void __init tegra_arthur_init(void)
 {
-	int err;
-	struct clk *c, *p;
-
 	tegra_init_suspend(&seaboard_suspend);
 
 	__init_debug_uart_B();
 
-	seaboard_audio_pdata.gpio_spkr_en = -1; /* No spkr_en on max98095. */
-	/* Set up the GPIO and pingroup controlling the camera's power. */
-	tegra_gpio_enable(TEGRA_GPIO_PV4);
-	err = gpio_request(TEGRA_GPIO_PV4, "cam_3v3_pwr_en");
-	WARN_ON(err);
-	err = gpio_direction_output(TEGRA_GPIO_PV4, 1);
-	WARN_ON(err);
-	tegra_pinmux_set_tristate(TEGRA_PINGROUP_GPV, TEGRA_TRI_NORMAL);
-	/* Export this GPIO so power can be controlled from userspace. */
-	err = gpio_export(TEGRA_GPIO_PV4, false);
-	WARN_ON(err);
-
 	seaboard_kbc_platform_data.keymap_data = &cros_keymap_data;
 
-	tegra_ehci1_device.dev.platform_data = &tegra_ehci_pdata[0];
-	tegra_ehci3_device.dev.platform_data = &tegra_ehci_pdata[2];
-	arthur_common_init();
-	arthur_panel_init();
-	arthur_emc_init();
-
-	/* Temporary hack to keep SDIO for wifi capped at 43.2MHz due to
-	 * stability issues with brcmfmac at 48MHz.
-	 */
-	c = tegra_get_clock_by_name("sdmmc1");
-	p = tegra_get_clock_by_name("pll_p");
-	if (c && p) {
-		clk_set_parent(c, p);
-		clk_set_rate(c, 43200000);
-		clk_enable(c);
-	}
+	seaboard_common_init();
 
 	arthur_i2c_register_devices();
 	seaboard_i2c_init();
@@ -1301,18 +1069,14 @@ static void __init tegra_asymptote_init(void)
 {
 	struct clk *c, *p;
 
+	seaboard_audio_pdata.gpio_hp_invert = 0;
 	tegra_init_suspend(&seaboard_suspend);
 
 	__init_debug_uart_B();
 
 	seaboard_kbc_platform_data.keymap_data = &cros_keymap_data;
 
-	tegra_ehci1_device.dev.platform_data = &tegra_ehci_pdata[0];
-	tegra_ehci2_device.dev.platform_data = &tegra_ehci_pdata[1];
-	tegra_ehci3_device.dev.platform_data = &tegra_ehci_pdata[2];
-
 	asymptote_common_init();
-	asymptote_panel_init();
 	/* asymptote has same memory config as seaboard (for now) */
 	seaboard_emc_init();
 
@@ -1365,10 +1129,6 @@ void __init tegra_ventana_init(void)
 
 	seaboard_gpio_keys_device.dev.platform_data = &ventana_keys_data;
 
-	tegra_ehci1_device.dev.platform_data = &tegra_ehci_pdata[0];
-	tegra_ehci2_device.dev.platform_data = &tegra_ehci_pdata[1];
-	tegra_ehci3_device.dev.platform_data = &tegra_ehci_pdata[2];
-
 	ventana_common_init();
 	seaboard_panel_init();
 	ventana_emc_init();
@@ -1389,11 +1149,11 @@ void __init tegra_common_reserve(void)
 		pr_warn("Cannot reserve first 4K of memory for safety\n");
 
 	/*
-	 * reserve 256MB for carveout, 1368*910*4*2 (=9959040) for fb_size,
+	 * reserve 128MB for carveout, 1368*910*4*2 (=9959040) for fb_size,
 	 * and 0 for fb2_size.
 	 */
 	fb_size = round_up((1368 * 910 * 4 * 2), PAGE_SIZE);
-	tegra_reserve(SZ_256M, fb_size, 0);
+	tegra_reserve(256 * 1024 * 1024, fb_size, 0);
 }
 
 static const char *seaboard_dt_board_compat[] = {
