@@ -22,11 +22,15 @@
 #include <linux/dma-mapping.h>
 #include <linux/fsl_devices.h>
 #include <linux/serial_8250.h>
+#include <linux/i2c-tegra.h>
+#include <linux/platform_data/tegra_usb.h>
 #include <asm/pmu.h>
 #include <mach/irqs.h>
 #include <mach/iomap.h>
 #include <mach/dma.h>
+#include <mach/usb_phy.h>
 #include <mach/nvhost.h>
+#include "gpio-names.h"
 
 static struct resource i2c_resource1[] = {
 	[0] = {
@@ -80,13 +84,29 @@ static struct resource i2c_resource4[] = {
 	},
 };
 
+static struct tegra_i2c_platform_data tegra_i2c1_platform_data = {
+	.bus_clk_rate   = { 400000 },
+};
+
+static struct tegra_i2c_platform_data tegra_i2c2_platform_data = {
+	.bus_clk_rate   = { 400000 },
+};
+
+static struct tegra_i2c_platform_data tegra_i2c3_platform_data = {
+	.bus_clk_rate   = { 400000 },
+};
+
+static struct tegra_i2c_platform_data tegra_dvc_platform_data = {
+	.bus_clk_rate   = { 400000 },
+};
+
 struct platform_device tegra_i2c_device1 = {
 	.name		= "tegra-i2c",
 	.id		= 0,
 	.resource	= i2c_resource1,
 	.num_resources	= ARRAY_SIZE(i2c_resource1),
 	.dev = {
-		.platform_data = 0,
+		.platform_data = &tegra_i2c1_platform_data,
 	},
 };
 
@@ -96,7 +116,7 @@ struct platform_device tegra_i2c_device2 = {
 	.resource	= i2c_resource2,
 	.num_resources	= ARRAY_SIZE(i2c_resource2),
 	.dev = {
-		.platform_data = 0,
+		.platform_data = &tegra_i2c2_platform_data,
 	},
 };
 
@@ -106,7 +126,7 @@ struct platform_device tegra_i2c_device3 = {
 	.resource	= i2c_resource3,
 	.num_resources	= ARRAY_SIZE(i2c_resource3),
 	.dev = {
-		.platform_data = 0,
+		.platform_data = &tegra_i2c3_platform_data,
 	},
 };
 
@@ -116,7 +136,7 @@ struct platform_device tegra_i2c_device4 = {
 	.resource	= i2c_resource4,
 	.num_resources	= ARRAY_SIZE(i2c_resource4),
 	.dev = {
-		.platform_data = 0,
+		.platform_data = &tegra_dvc_platform_data,
 	},
 };
 
@@ -296,19 +316,6 @@ struct platform_device tegra_sdhci_device4 = {
 	.num_resources	= ARRAY_SIZE(sdhci_resource4),
 };
 
-static struct resource tegra_udc_resources[] = {
-	[0] = {
-		.start	= TEGRA_USB_BASE,
-		.end	= TEGRA_USB_BASE + TEGRA_USB_SIZE - 1,
-		.flags	= IORESOURCE_MEM,
-	},
-	[1] = {
-		.start	= INT_USB,
-		.end	= INT_USB,
-		.flags	= IORESOURCE_IRQ,
-	},
-};
-
 static struct resource tegra_usb1_resources[] = {
 	[0] = {
 		.start	= TEGRA_USB_BASE,
@@ -348,25 +355,26 @@ static struct resource tegra_usb3_resources[] = {
 	},
 };
 
-static u64 tegra_udc_dmamask = DMA_BIT_MASK(32);
-
-/* When the bus reset is seen on Tegra, the PORTSCX_PORT_RESET bit
- * is not set */
-static struct fsl_usb2_platform_data tegra_udc_pdata = {
-	.operating_mode	= FSL_USB2_DR_DEVICE,
-	.phy_mode	= FSL_USB2_PHY_UTMI,
+static struct tegra_ulpi_config tegra_ehci2_ulpi_phy_config = {
+	/* All existing boards use GPIO PV0 for phy reset */
+	.reset_gpio = TEGRA_GPIO_PV0,
+	.clk = "cdev2",
 };
 
-struct platform_device tegra_udc_device = {
-	.name	= "fsl-tegra-udc",
-	.id	= -1,
-	.dev	= {
-		.dma_mask	= &tegra_udc_dmamask,
-		.coherent_dma_mask = DMA_BIT_MASK(32),
-		.platform_data	= &tegra_udc_pdata,
-	},
-	.resource = tegra_udc_resources,
-	.num_resources = ARRAY_SIZE(tegra_udc_resources),
+static struct tegra_ehci_platform_data tegra_ehci1_pdata = {
+	.operating_mode = TEGRA_USB_OTG,
+	.power_down_on_bus_suspend = 1,
+};
+
+static struct tegra_ehci_platform_data tegra_ehci2_pdata = {
+	.phy_config = &tegra_ehci2_ulpi_phy_config,
+	.operating_mode = TEGRA_USB_HOST,
+	.power_down_on_bus_suspend = 1,
+};
+
+static struct tegra_ehci_platform_data tegra_ehci3_pdata = {
+	.operating_mode = TEGRA_USB_HOST,
+	.power_down_on_bus_suspend = 1,
 };
 
 static u64 tegra_ehci_dmamask = DMA_BIT_MASK(32);
@@ -377,6 +385,7 @@ struct platform_device tegra_ehci1_device = {
 	.dev	= {
 		.dma_mask	= &tegra_ehci_dmamask,
 		.coherent_dma_mask = DMA_BIT_MASK(32),
+		.platform_data = &tegra_ehci1_pdata,
 	},
 	.resource = tegra_usb1_resources,
 	.num_resources = ARRAY_SIZE(tegra_usb1_resources),
@@ -388,6 +397,7 @@ struct platform_device tegra_ehci2_device = {
 	.dev	= {
 		.dma_mask	= &tegra_ehci_dmamask,
 		.coherent_dma_mask = DMA_BIT_MASK(32),
+		.platform_data = &tegra_ehci2_pdata,
 	},
 	.resource = tegra_usb2_resources,
 	.num_resources = ARRAY_SIZE(tegra_usb2_resources),
@@ -399,6 +409,7 @@ struct platform_device tegra_ehci3_device = {
 	.dev	= {
 		.dma_mask	= &tegra_ehci_dmamask,
 		.coherent_dma_mask = DMA_BIT_MASK(32),
+		.platform_data = &tegra_ehci3_pdata,
 	},
 	.resource = tegra_usb3_resources,
 	.num_resources = ARRAY_SIZE(tegra_usb3_resources),
@@ -539,26 +550,6 @@ struct platform_device tegra_uarte_device = {
 	},
 };
 
-static struct resource tegra_otg_resources[] = {
-	[0] = {
-		.start	= TEGRA_USB_BASE,
-		.end	= TEGRA_USB_BASE + TEGRA_USB_SIZE - 1,
-		.flags	= IORESOURCE_MEM,
-	},
-	[1] = {
-		.start	= INT_USB,
-		.end	= INT_USB,
-		.flags	= IORESOURCE_IRQ,
-	},
-};
-
-struct platform_device tegra_otg_device = {
-	.name		= "tegra-otg",
-	.id		= -1,
-	.resource	= tegra_otg_resources,
-	.num_resources	= ARRAY_SIZE(tegra_otg_resources),
-};
-
 static struct resource i2s_resource1[] = {
 	[0] = {
 		.start	= INT_I2S1,
@@ -654,72 +645,94 @@ struct platform_device tegra_pcm_device = {
 	.id = -1,
 };
 
-static struct resource w1_resources[] = {
+static struct resource tegra_kbc_resources[] = {
+       [0] = {
+               .start = TEGRA_KBC_BASE,
+               .end   = TEGRA_KBC_BASE + TEGRA_KBC_SIZE - 1,
+               .flags = IORESOURCE_MEM,
+       },
+       [1] = {
+               .start = INT_KBC,
+               .end   = INT_KBC,
+               .flags = IORESOURCE_IRQ,
+       },
+};
+
+struct platform_device tegra_kbc_device = {
+       .name		= "tegra-kbc",
+       .id		= -1,
+       .resource	= tegra_kbc_resources,
+       .num_resources	= ARRAY_SIZE(tegra_kbc_resources),
+};
+
+static struct resource tegra_rtc_resources[] = {
 	[0] = {
-		.start = INT_OWR,
-		.end   = INT_OWR,
-		.flags = IORESOURCE_IRQ
-	},
-	[1] = {
-		.start = TEGRA_OWR_BASE,
-		.end = TEGRA_OWR_BASE + TEGRA_OWR_SIZE - 1,
-		.flags = IORESOURCE_MEM
-	}
-};
-
-struct platform_device tegra_w1_device = {
-	.name          = "tegra_w1",
-	.id            = -1,
-	.resource      = w1_resources,
-	.num_resources = ARRAY_SIZE(w1_resources),
-};
-
-static struct resource tegra_gart_resources[] = {
-	[0] = {
-		.name	= "mc",
-		.flags	= IORESOURCE_MEM,
-		.start	= TEGRA_MC_BASE,
-		.end	= TEGRA_MC_BASE + TEGRA_MC_SIZE - 1,
-	},
-	[1] = {
-		.name	= "gart",
-		.flags	= IORESOURCE_MEM,
-		.start	= TEGRA_GART_BASE,
-		.end	= TEGRA_GART_BASE + TEGRA_GART_SIZE - 1,
-	}
-};
-
-struct platform_device tegra_gart_device = {
-	.name		= "tegra_gart",
-	.id		= -1,
-	.num_resources	= ARRAY_SIZE(tegra_gart_resources),
-	.resource	= tegra_gart_resources
-};
-
-#define CLK_RESET_RST_SOURCE	0x0
-static struct resource tegra_wdt_resources[] = {
-	[0] = {
-		.start	= TEGRA_CLK_RESET_BASE + CLK_RESET_RST_SOURCE,
-		.end	= TEGRA_CLK_RESET_BASE + CLK_RESET_RST_SOURCE + 4 - 1,
+		.start	= TEGRA_RTC_BASE,
+		.end	= TEGRA_RTC_BASE + TEGRA_RTC_SIZE - 1,
 		.flags	= IORESOURCE_MEM,
 	},
 	[1] = {
-		.start	= TEGRA_TMR1_BASE,
-		.end	= TEGRA_TMR1_BASE + TEGRA_TMR1_SIZE - 1,
-		.flags	= IORESOURCE_MEM,
-	},
-	[2] = {
-		.start	= INT_TMR1,
-		.end	= INT_TMR1,
+		.start	= INT_RTC,
+		.end	= INT_RTC,
 		.flags	= IORESOURCE_IRQ,
 	},
 };
 
-struct platform_device tegra_wdt_device = {
-	.name		= "tegra_wdt",
+struct platform_device tegra_rtc_device = {
+	.name		= "tegra_rtc",
 	.id		= -1,
-	.num_resources	= ARRAY_SIZE(tegra_wdt_resources),
-	.resource	= tegra_wdt_resources,
+	.resource	= tegra_rtc_resources,
+	.num_resources	= ARRAY_SIZE(tegra_rtc_resources),
+};
+
+static struct resource tegra_grhost_resources[] = {
+	{
+		.start = TEGRA_HOST1X_BASE,
+		.end = TEGRA_HOST1X_BASE + TEGRA_HOST1X_SIZE - 1,
+		.flags = IORESOURCE_MEM,
+	},
+	{
+		.start = TEGRA_DISPLAY_BASE,
+		.end = TEGRA_DISPLAY_BASE + TEGRA_DISPLAY_SIZE - 1,
+		.flags = IORESOURCE_MEM,
+	},
+	{
+		.start = TEGRA_DISPLAY2_BASE,
+		.end = TEGRA_DISPLAY2_BASE + TEGRA_DISPLAY2_SIZE - 1,
+		.flags = IORESOURCE_MEM,
+	},
+	{
+		.start = TEGRA_VI_BASE,
+		.end = TEGRA_VI_BASE + TEGRA_VI_SIZE - 1,
+		.flags = IORESOURCE_MEM,
+	},
+	{
+		.start = TEGRA_ISP_BASE,
+		.end = TEGRA_ISP_BASE + TEGRA_ISP_SIZE - 1,
+		.flags = IORESOURCE_MEM,
+	},
+	{
+		.start = TEGRA_MPE_BASE,
+		.end = TEGRA_MPE_BASE + TEGRA_MPE_SIZE - 1,
+		.flags = IORESOURCE_MEM,
+	},
+	{
+		.start = INT_SYNCPT_THRESH_BASE,
+		.end = INT_SYNCPT_THRESH_BASE + INT_SYNCPT_THRESH_NR - 1,
+		.flags = IORESOURCE_IRQ,
+	},
+	{
+		.start = INT_HOST1X_MPCORE_GENERAL,
+		.end = INT_HOST1X_MPCORE_GENERAL,
+		.flags = IORESOURCE_IRQ,
+	},
+};
+
+struct platform_device tegra_grhost_device = {
+	.name = "tegra_grhost",
+	.id = -1,
+	.resource = tegra_grhost_resources,
+	.num_resources = ARRAY_SIZE(tegra_grhost_resources),
 };
 
 static struct resource tegra_pwfm0_resource = {
@@ -774,134 +787,26 @@ struct platform_device tegra_pwfm3_device = {
 	.resource	= &tegra_pwfm3_resource,
 };
 
-static struct resource tegra_grhost_resources[] = {
-	{
-		.start = TEGRA_HOST1X_BASE,
-		.end = TEGRA_HOST1X_BASE + TEGRA_HOST1X_SIZE - 1,
-		.flags = IORESOURCE_MEM,
-	},
-	{
-		.start = TEGRA_DISPLAY_BASE,
-		.end = TEGRA_DISPLAY_BASE + TEGRA_DISPLAY_SIZE - 1,
-		.flags = IORESOURCE_MEM,
-	},
-	{
-		.start = TEGRA_DISPLAY2_BASE,
-		.end = TEGRA_DISPLAY2_BASE + TEGRA_DISPLAY2_SIZE - 1,
-		.flags = IORESOURCE_MEM,
-	},
-	{
-		.start = TEGRA_VI_BASE,
-		.end = TEGRA_VI_BASE + TEGRA_VI_SIZE - 1,
-		.flags = IORESOURCE_MEM,
-	},
-	{
-		.start = TEGRA_ISP_BASE,
-		.end = TEGRA_ISP_BASE + TEGRA_ISP_SIZE - 1,
-		.flags = IORESOURCE_MEM,
-	},
-	{
-		.start = TEGRA_MPE_BASE,
-		.end = TEGRA_MPE_BASE + TEGRA_MPE_SIZE - 1,
-		.flags = IORESOURCE_MEM,
-	},
-	{
-		.start = INT_SYNCPT_THRESH_BASE,
-		.end = INT_SYNCPT_THRESH_BASE + INT_SYNCPT_THRESH_NR - 1,
-		.flags = IORESOURCE_IRQ,
-	},
-	{
-		.start = INT_HOST1X_MPCORE_GENERAL,
-		.end = INT_HOST1X_MPCORE_GENERAL,
-		.flags = IORESOURCE_IRQ,
-	},
-};
-
-struct platform_device tegra_grhost_device = {
-	.name = "tegra_grhost",
-	.id = -1,
-	.resource = tegra_grhost_resources,
-	.num_resources = ARRAY_SIZE(tegra_grhost_resources),
-};
-
-static struct resource tegra_avp_resources[] = {
+static struct resource tegra_gart_resources[] = {
 	[0] = {
-		.start	= INT_SHR_SEM_INBOX_IBF,
-		.end	= INT_SHR_SEM_INBOX_IBF,
-		.flags	= IORESOURCE_IRQ,
-		.name	= "mbox_from_avp_pending",
-	},
-};
-
-struct platform_device tegra_avp_device = {
-	.name		= "tegra-avp",
-	.id		= -1,
-	.num_resources	= ARRAY_SIZE(tegra_avp_resources),
-	.resource	= tegra_avp_resources,
-	.dev  = {
-		.coherent_dma_mask	= 0xffffffffULL,
-	},
-};
-
-static struct resource tegra_aes_resources[] = {
-	{
-		.start	= TEGRA_VDE_BASE,
-		.end	= TEGRA_VDE_BASE + TEGRA_VDE_SIZE - 1,
-		.flags	= IORESOURCE_MEM,
-	},
-};
-
-static u64 tegra_aes_dma_mask = DMA_BIT_MASK(32);
-
-struct platform_device tegra_aes_device = {
-	.name		= "tegra-aes",
-	.id		= -1,
-	.resource	= tegra_aes_resources,
-	.num_resources	= ARRAY_SIZE(tegra_aes_resources),
-	.dev	= {
-		.dma_mask = &tegra_aes_dma_mask,
-		.coherent_dma_mask = DMA_BIT_MASK(32),
-	},
-};
-
-static struct resource tegra_kbc_resources[] = {
-       [0] = {
-               .start = TEGRA_KBC_BASE,
-               .end   = TEGRA_KBC_BASE + TEGRA_KBC_SIZE - 1,
-               .flags = IORESOURCE_MEM,
-       },
-       [1] = {
-               .start = INT_KBC,
-               .end   = INT_KBC,
-               .flags = IORESOURCE_IRQ,
-       },
-};
-
-struct platform_device tegra_kbc_device = {
-       .name		= "tegra-kbc",
-       .id		= -1,
-       .resource	= tegra_kbc_resources,
-       .num_resources	= ARRAY_SIZE(tegra_kbc_resources),
-};
-
-static struct resource tegra_rtc_resources[] = {
-	[0] = {
-		.start	= TEGRA_RTC_BASE,
-		.end	= TEGRA_RTC_BASE + TEGRA_RTC_SIZE - 1,
+		.name	= "mc",
 		.flags	= IORESOURCE_MEM,
+		.start	= TEGRA_MC_BASE,
+		.end	= TEGRA_MC_BASE + TEGRA_MC_SIZE - 1,
 	},
 	[1] = {
-		.start	= INT_RTC,
-		.end	= INT_RTC,
-		.flags	= IORESOURCE_IRQ,
-	},
+		.name	= "gart",
+		.flags	= IORESOURCE_MEM,
+		.start	= TEGRA_GART_BASE,
+		.end	= TEGRA_GART_BASE + TEGRA_GART_SIZE - 1,
+	}
 };
 
-struct platform_device tegra_rtc_device = {
-	.name		= "tegra_rtc",
+struct platform_device tegra_gart_device = {
+	.name		= "tegra_gart",
 	.id		= -1,
-	.resource	= tegra_rtc_resources,
-	.num_resources	= ARRAY_SIZE(tegra_rtc_resources),
+	.num_resources	= ARRAY_SIZE(tegra_gart_resources),
+	.resource	= tegra_gart_resources
 };
 
 static struct resource tegra_camera_resources[] = {
@@ -925,3 +830,22 @@ struct nvhost_device tegra_camera_device = {
 	.num_resources	= ARRAY_SIZE(tegra_camera_resources),
 	.resource	= tegra_camera_resources,
 };
+
+static struct resource tegra_avp_resources[] = {
+	[0] = {
+		.start	= INT_SHR_SEM_INBOX_IBF,
+		.end	= INT_SHR_SEM_INBOX_IBF,
+		.flags	= IORESOURCE_IRQ,
+		.name	= "mbox_from_avp_pending",
+	},
+};
+
+struct platform_device tegra_avp_device = {
+	.name		= "tegra-avp",
+	.id		= -1,
+	.num_resources	= ARRAY_SIZE(tegra_avp_resources),
+	.resource	= tegra_avp_resources,
+	.dev  = {
+		.coherent_dma_mask	= 0xffffffffULL,
+	},
+};
