@@ -26,6 +26,7 @@
 #include <linux/delay.h>
 #include <linux/slab.h>
 #include <linux/i2c-tegra.h>
+#include <linux/of_i2c.h>
 
 #include <asm/unaligned.h>
 
@@ -47,7 +48,6 @@
 #define I2C_SL_CNFG_NEWSL			(1<<2)
 #define I2C_SL_ADDR1				0x02c
 #define I2C_SL_ADDR2				0x030
-#define I2C_SL_DELAY_COUNT			0x03c
 #define I2C_TX_FIFO				0x050
 #define I2C_RX_FIFO				0x054
 #define I2C_PACKET_TRANSFER_STATUS		0x058
@@ -103,9 +103,6 @@
 #define I2C_HEADER_MASTER_ADDR_SHIFT		12
 #define I2C_HEADER_SLAVE_ADDR_SHIFT		1
 
-#define SL_ADDR1(addr) (addr & 0xff)
-#define SL_ADDR2(addr) ((addr >> 8) & 0xff)
-
 static void dvc_writel(struct tegra_i2c_dev *i2c_dev, u32 val, unsigned long reg)
 {
 	writel(val, i2c_dev->base + reg);
@@ -165,11 +162,6 @@ static void tegra_i2c_unmask_irq(struct tegra_i2c_dev *i2c_dev, u32 mask)
 	i2c_writel(i2c_dev, int_mask, I2C_INT_MASK);
 }
 
-static void tegra_i2c_set_clk(struct tegra_i2c_dev *i2c_dev, unsigned int freq)
-{
-	clk_set_rate(i2c_dev->clk, freq * 8);
-}
-
 static int tegra_i2c_flush_fifos(struct tegra_i2c_dev *i2c_dev)
 {
 	unsigned long timeout = jiffies + HZ;
@@ -308,28 +300,6 @@ static void tegra_dvc_init(struct tegra_i2c_dev *i2c_dev)
 	dvc_writel(i2c_dev, val, DVC_CTRL_REG1);
 }
 
-static void tegra_i2c_slave_init(struct tegra_i2c_dev *i2c_dev)
-{
-	u32 val = i2c_readl(i2c_dev, I2C_SL_CNFG);
-
-	val |= I2C_SL_CNFG_NEWSL;
-
-	if(!i2c_dev->is_slave)
-		val |= I2C_SL_CNFG_NACK;
-
-	i2c_writel(i2c_dev, val, I2C_SL_CNFG);
-
-	if (i2c_dev->is_slave)
-		i2c_writel(i2c_dev, 0x1E, I2C_SL_DELAY_COUNT);
-
-	if (i2c_dev->slave_addr) {
-		u16 addr = i2c_dev->slave_addr >> 1;
-
-		i2c_writel(i2c_dev, SL_ADDR1(addr), I2C_SL_ADDR1);
-		i2c_writel(i2c_dev, SL_ADDR2(addr), I2C_SL_ADDR2);
-	}
-}
-
 static int tegra_i2c_init(struct tegra_i2c_dev *i2c_dev)
 {
 	u32 val;
@@ -344,13 +314,23 @@ static int tegra_i2c_init(struct tegra_i2c_dev *i2c_dev)
 	if (i2c_dev->is_dvc)
 		tegra_dvc_init(i2c_dev);
 
-	val = I2C_CNFG_NEW_MASTER_FSM | I2C_CNFG_PACKET_MODE_EN | (0x2 << I2C_CNFG_DEBOUNCE_CNT_SHIFT);
+	val = I2C_CNFG_NEW_MASTER_FSM | I2C_CNFG_PACKET_MODE_EN |
+		(0x2 << I2C_CNFG_DEBOUNCE_CNT_SHIFT);
 	i2c_writel(i2c_dev, val, I2C_CNFG);
 	i2c_writel(i2c_dev, 0, I2C_INT_MASK);
-	tegra_i2c_set_clk(i2c_dev, i2c_dev->last_bus_clk);
+	clk_set_rate(i2c_dev->clk, i2c_dev->last_bus_clk_rate * 8);
 
-	if (!i2c_dev->is_dvc)
-		tegra_i2c_slave_init(i2c_dev);
+	if (!i2c_dev->is_dvc) {
+		u32 sl_cfg = i2c_readl(i2c_dev, I2C_SL_CNFG);
+		u16 sl_addr = i2c_dev->is_slave ? i2c_dev->slave_addr >> I2C_HEADER_SLAVE_ADDR_SHIFT : 0xfc;
+		sl_cfg |= I2C_SL_CNFG_NEWSL;
+		if (!i2c_dev->is_slave)
+			sl_cfg |= I2C_SL_CNFG_NACK;
+		i2c_writel(i2c_dev, sl_cfg, I2C_SL_CNFG);
+		i2c_writel(i2c_dev, sl_addr & 0xff, I2C_SL_ADDR1);
+		i2c_writel(i2c_dev, (sl_addr >> 8) & 0xff, I2C_SL_ADDR2);
+
+	}
 
 	val = 7 << I2C_FIFO_CONTROL_TX_TRIG_SHIFT |
 		0 << I2C_FIFO_CONTROL_RX_TRIG_SHIFT;
@@ -441,7 +421,7 @@ static irqreturn_t tegra_i2c_isr(int irq, void *dev_id)
 		dvc_writel(i2c_dev, DVC_STATUS_I2C_DONE_INTR, DVC_STATUS);
 	return IRQ_HANDLED;
 err:
-	/* An error occured, mask all interrupts */
+	/* An error occurred, mask all interrupts */
 	tegra_i2c_mask_irq(i2c_dev, I2C_INT_NO_ACK | I2C_INT_ARBITRATION_LOST |
 		I2C_INT_PACKET_XFER_COMPLETE | I2C_INT_TX_FIFO_DATA_REQ |
 		I2C_INT_RX_FIFO_DATA_REQ);
@@ -468,7 +448,6 @@ static int tegra_i2c_xfer_msg(struct tegra_i2c_bus *i2c_bus,
 	i2c_dev->msg_buf = msg->buf;
 	i2c_dev->msg_buf_remaining = msg->len;
 	i2c_dev->msg_err = I2C_ERR_NONE;
-	i2c_dev->msg_transfer_complete = 0;
 	i2c_dev->msg_read = (msg->flags & I2C_M_RD);
 	INIT_COMPLETION(i2c_dev->msg_complete);
 
@@ -549,16 +528,16 @@ static int tegra_i2c_xfer(struct i2c_adapter *adap, struct i2c_msg msgs[],
 
 	if (i2c_dev->last_mux != i2c_bus->mux) {
 		tegra_pinmux_set_safe_pinmux_table(i2c_dev->last_mux,
-						   i2c_dev->last_mux_len);
+			i2c_dev->last_mux_len);
 		tegra_pinmux_config_pinmux_table(i2c_bus->mux,
-						 i2c_bus->mux_len);
+			i2c_bus->mux_len);
 		i2c_dev->last_mux = i2c_bus->mux;
 		i2c_dev->last_mux_len = i2c_bus->mux_len;
 	}
 
-	if (i2c_dev->last_bus_clk != i2c_bus->bus_clk_rate) {
-		tegra_i2c_set_clk(i2c_dev, i2c_bus->bus_clk_rate);
-		i2c_dev->last_bus_clk = i2c_bus->bus_clk_rate;
+	if (i2c_dev->last_bus_clk_rate != i2c_bus->bus_clk_rate) {
+		clk_set_rate(i2c_dev->clk, i2c_bus->bus_clk_rate * 8);
+		i2c_dev->last_bus_clk_rate = i2c_bus->bus_clk_rate;
 	}
 
 	clk_enable(i2c_dev->clk);
@@ -588,30 +567,30 @@ static const struct i2c_algorithm tegra_i2c_algo = {
 static int tegra_i2c_probe(struct platform_device *pdev)
 {
 	struct tegra_i2c_dev *i2c_dev;
-	struct tegra_i2c_platform_data *pdata = pdev->dev.platform_data;
+	struct tegra_i2c_platform_data *plat = pdev->dev.platform_data;
 	struct resource *res;
 	struct resource *iomem;
 	struct clk *clk;
 	struct clk *i2c_clk;
+	const unsigned int *prop;
 	void *base;
 	int irq;
 	int nbus;
 	int i = 0;
 	int ret = 0;
 
-	if (!pdata) {
+	if (!plat) {
 		dev_err(&pdev->dev, "no platform data?\n");
 		return -ENODEV;
 	}
 
-
-	if (pdata->bus_count <= 0 || pdata->adapter_nr < 0) {
+	if (plat->bus_count <= 0 || plat->adapter_nr < 0) {
 		dev_err(&pdev->dev, "invalid platform data?\n");
 		return -ENODEV;
 	}
 
-	WARN_ON(pdata->bus_count > TEGRA_I2C_MAX_BUS);
-	nbus = min(TEGRA_I2C_MAX_BUS, pdata->bus_count);
+	WARN_ON(plat->bus_count > TEGRA_I2C_MAX_BUS);
+	nbus = min(TEGRA_I2C_MAX_BUS, plat->bus_count);
 
 	res = platform_get_resource(pdev, IORESOURCE_MEM, 0);
 	if (!res) {
@@ -654,7 +633,6 @@ static int tegra_i2c_probe(struct platform_device *pdev)
 
 	i2c_dev = kzalloc(sizeof(struct tegra_i2c_dev) +
 			  (nbus-1) * sizeof(struct tegra_i2c_bus), GFP_KERNEL);
-
 	if (!i2c_dev) {
 		ret = -ENOMEM;
 		goto err_i2c_clk_put;
@@ -667,12 +645,13 @@ static int tegra_i2c_probe(struct platform_device *pdev)
 	i2c_dev->irq = irq;
 	i2c_dev->cont_id = pdev->id;
 	i2c_dev->dev = &pdev->dev;
-	i2c_dev->last_bus_clk = pdata->bus_clk_rate[0] ?: 100000;
+	i2c_dev->last_bus_clk_rate = (plat && plat->bus_clk_rate[0]) ?: 100000;
+
 	rt_mutex_init(&i2c_dev->dev_lock);
 
-	i2c_dev->slave_addr = pdata->slave_addr;
-	i2c_dev->is_dvc = pdata->is_dvc;
-	i2c_dev->is_slave = pdata->is_slave;
+	i2c_dev->is_dvc = plat->is_dvc;
+	i2c_dev->is_slave = plat->is_slave;
+	i2c_dev->slave_addr = plat->slave_addr;
 	init_completion(&i2c_dev->msg_complete);
 
 	platform_set_drvdata(pdev, i2c_dev);
@@ -696,29 +675,39 @@ static int tegra_i2c_probe(struct platform_device *pdev)
 		struct tegra_i2c_bus *i2c_bus = &i2c_dev->busses[i];
 
 		i2c_bus->dev = i2c_dev;
-		i2c_bus->mux = pdata->bus_mux[i];
-		i2c_bus->mux_len = pdata->bus_mux_len[i];
-		i2c_bus->bus_clk_rate = pdata->bus_clk_rate[i] ?: 100000;
+		i2c_bus->mux = plat->bus_mux[i];
+		i2c_bus->mux_len = plat->bus_mux_len[i];
+		if (i2c_dev->dev->of_node) {    /* if there is a device tree node ... */
+			prop = of_get_property(i2c_dev->dev->of_node,
+					"clock-frequency", NULL);
+			if (prop)
+				i2c_bus->bus_clk_rate = be32_to_cpup(prop);
+		} else 
+			i2c_bus->bus_clk_rate = plat->bus_clk_rate[i] ?: 100000;
 
-		i2c_bus->adapter.algo = &tegra_i2c_algo;
 		i2c_set_adapdata(&i2c_bus->adapter, i2c_bus);
 		i2c_bus->adapter.owner = THIS_MODULE;
 		i2c_bus->adapter.class = I2C_CLASS_HWMON;
 		strlcpy(i2c_bus->adapter.name, "Tegra I2C adapter",
 			sizeof(i2c_bus->adapter.name));
+		i2c_bus->adapter.algo = &tegra_i2c_algo;
 		i2c_bus->adapter.dev.parent = &pdev->dev;
-		i2c_bus->adapter.nr = pdata->adapter_nr + i;
+		i2c_bus->adapter.nr = plat->adapter_nr + i;
+		i2c_bus->adapter.dev.of_node = pdev->dev.of_node;
 		i2c_bus->adapter.retries = TEGRA_I2C_RETRIES;
-
+	
 		ret = i2c_add_numbered_adapter(&i2c_bus->adapter);
 		if (ret) {
 			dev_err(&pdev->dev, "Failed to add I2C adapter\n");
 			goto err_del_bus;
 		}
 		i2c_dev->bus_count++;
+		of_i2c_register_devices(&i2c_bus->adapter);
 	}
 
+
 	return 0;
+
 err_del_bus:
 	while (i2c_dev->bus_count--)
 		i2c_del_adapter(&i2c_dev->busses[i2c_dev->bus_count].adapter);
