@@ -1745,7 +1745,7 @@ static void cyapa_probe_detect_work_handler(struct work_struct *work)
 	}
 
 	cyapa->irq = client->irq;
-	set_irq_type(cyapa->irq, IRQF_TRIGGER_FALLING);
+	irq_set_irq_type(cyapa->irq, IRQF_TRIGGER_FALLING);
 	ret = request_irq(cyapa->irq,
 			cyapa_irq,
 			0,
